@@ -11,10 +11,25 @@ module.exports = {
   themeConfig: {
     prism: {
       additionalLanguages: ['hcl'],
+      magicComments: [
+        {
+          className: 'theme-code-block-highlighted-line',
+          line: 'highlight-next-line',
+          block: {start: 'highlight-start', end: 'highlight-end'},
+        },
+        {
+          className: 'code-block-error-message',
+          line: 'highlight-next-line-error-message',
+        },
+        {
+          className: 'code-block-info-line',
+          line: 'highlight-next-line-info',
+        },
+      ],
     },
     algolia: {
       appId: 'D158RRDJO1',
-      apiKey: "383b9efbb1486f812c8c023556b15396",
+      apiKey: "3585a7e658ef4ab7775a9099be3778d2",
       indexName: "atlasgo",
     },
     navbar: {
@@ -44,8 +59,8 @@ module.exports = {
           position: 'left',
         },
         {
-          to: 'https://pkg.go.dev/ariga.io/atlas',
-          label: 'GoDoc',
+          to: 'knowledge',
+          label: 'Knowledge Center',
           position: 'left',
         },
         {
@@ -62,10 +77,12 @@ module.exports = {
           "title": "Docs",
           "items": [
             {"label": "Getting Started", "to": "cli/getting-started/setting-up"},
-            {"label": "Data Definition Language ", "to": "ddl/intro"},
-            {"label": "CLI Reference", "to": "cli/reference"},
+            {"label": "Data Definition Language ", "to": "concepts/ddl"},
+            {"label": "CLI Reference", "to": "cli-reference"},
             {"label": "Blog", "to": "blog"},
+            {"label": "Knowledge Center", "to": "knowledge"},
             {"label": "About", "to": "about"},
+            {"label": "GoDoc", "to": "https://pkg.go.dev/ariga.io/atlas"},
           ]
         },
         {
@@ -73,6 +90,7 @@ module.exports = {
           "items": [
             {"label": "GitHub", "to": "https://github.com/ariga/atlas"},
             {"label": "Discord", "to": "https://discord.gg/zZ6sWVg6NT"},
+            {"label": "Twitter", "to": "https://twitter.com/ariga_io"}
           ]
         },
         {
@@ -105,8 +123,44 @@ module.exports = {
       {
         redirects: [
           {
-            to: '/dev-database',
-            from: '/cli/dev-database',
+            to: '/integrations/terraform-provider',
+            from: '/terraform-provider'
+          },
+          {
+            to: '/integrations/go-api',
+            from: ['/go-api/intro','/go-api/inspect'],
+          },
+          {
+            to: '/cli-reference',
+            from: '/cli/reference',
+          },
+          {
+            to: '/concepts/url',
+            from: '/cli/url',
+          },
+          {
+            to: '/concepts/dev-database',
+            from: '/dev-database',
+          },
+          {
+            to: '/concepts/ddl',
+            from: '/ddl/intro',
+          },
+          {
+            to: '/atlas-schema/input-variables',
+            from: '/ddl/input-variables',
+          },
+          {
+            to: '/atlas-schema/projects',
+            from: '/cli/projects',
+          },
+          {
+            to: '/atlas-schema/sql-types',
+            from: '/ddl/sql-types',
+          },
+          {
+            to: '/atlas-schema/sql-resources',
+            from: '/ddl/sql',
           },
         ],
       },
@@ -134,6 +188,8 @@ module.exports = {
         },
         blog: {
           showReadingTime: true,
+          blogSidebarTitle: 'All our posts',
+          blogSidebarCount: 'ALL',
         },
         theme: {
           customCss: require.resolve('./src/css/custom.css'),
