@@ -1,18 +1,13 @@
 import React from 'react';
 import Link from '@docusaurus/Link';
-import LayoutProviders from '@theme/LayoutProviders';
+import LayoutProvider from '@theme/Layout/Provider';
 import Footer from '@theme/Footer';
 import index from './index.module.css';
 import GithubIcon from '../assets/icons/github.svg';
 import DiscordIcon from '../assets/icons/discord.svg';
+import TwitterIcon from '../assets/icons/twitter.svg';
 import BrowserOnly from "@docusaurus/core/lib/client/exports/BrowserOnly";
 
-function Chip({ text }) {
-    return <div className={index.chip}>
-        {text}
-    </div>
-}
-
 function Nav() {
     return  <ul className={index.nav}>
         <li className={index.linkItem}>
@@ -21,8 +16,8 @@ function Nav() {
             </Link>
         </li>
         <li className={index.linkItem}>
-            <Link to="https://pkg.go.dev/ariga.io/atlas">
-                GoDoc
+            <Link to="/knowledge">
+                Knowledge Center
             </Link>
         </li>
         <li className={index.linkItem}>
@@ -42,6 +37,9 @@ function SocialLinks() {
         <a href="https://discord.gg/zZ6sWVg6NT" target="_blank">
             <DiscordIcon />
         </a>
+        <a href="https://twitter.com/ariga_io" target="_blank">
+            <TwitterIcon />
+        </a>
     </div>
 }
 
@@ -61,7 +59,7 @@ function AtlasButton({ link, text, type, style }) {
 }
 
 export default function () {
-    return <LayoutProviders>
+    return <LayoutProvider>
         {/* first slide */}
         <div id="slide1" className={index.slide1} style={{backgroundImage:'url(https://atlasgo.io/uploads/landing/background.png)'}}>
             <Header />
@@ -113,19 +111,6 @@ export default function () {
                         return <img style={{margin: "20px 0" }} src={erdImage} alt="erd"/>
                     }}
                 </BrowserOnly>
-
-                <section className={index.section} style={{ marginTop: "10px" }}>
-                   <h2 className={index.titleSecondary} style={{ textAlign: "center" }}>
-                       Visualize your schemas <br/> using the <span style={{color: "#2064E9"}}>Atlas UI</span>
-                   </h2>
-                   <p className={index.paragraphSecondary}
-                      style={{ color: "#757889" }}>
-                       With a single command-line flag, Atlas opens up as a rich user interface to visualize and manage your data topology.
-                   </p>
-               </section>
-                <AtlasButton text="Get Started" link="/ui/intro" type="secondaryButton"/>
-
-                <img  className={index.dashboardImage} src="https://atlasgo.io/uploads/atlas-ui-180122.png" alt="ent"/>
             </div>
         </div>
 
@@ -169,5 +154,5 @@ export default function () {
             </div>
         </div>
         <Footer />
-    </LayoutProviders>
+    </LayoutProvider>
 }
