@@ -7,7 +7,7 @@ package hclsqlspec
 import (
 	"testing"
 
-	"ariga.io/atlas/schema/schemaspec"
+	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
@@ -19,23 +19,23 @@ import (
 
 var dialects = []struct {
 	name string
-	schemaspec.Marshaler
-	schemaspec.Unmarshaler
+	schemahcl.Marshaler
+	Eval func(b []byte, v interface{}, inp map[string]string) error
 }{
 	{
-		name:        "mysql",
-		Marshaler:   mysql.MarshalHCL,
-		Unmarshaler: mysql.UnmarshalHCL,
+		name:      "mysql",
+		Marshaler: mysql.MarshalHCL,
+		Eval:      mysql.EvalHCLBytes,
 	},
 	{
-		name:        "postgres",
-		Marshaler:   postgres.MarshalHCL,
-		Unmarshaler: postgres.UnmarshalHCL,
+		name:      "postgres",
+		Marshaler: postgres.MarshalHCL,
+		Eval:      postgres.EvalHCLBytes,
 	},
 	{
-		name:        "sqlite",
-		Marshaler:   sqlite.MarshalHCL,
-		Unmarshaler: sqlite.UnmarshalHCL,
+		name:      "sqlite",
+		Marshaler: sqlite.MarshalHCL,
+		Eval:      sqlite.EvalHCLBytes,
 	},
 }
 
@@ -140,35 +140,35 @@ table "accounts" {
 		Tables: []*sqlspec.Table{
 			{
 				Name:   "users",
-				Schema: &schemaspec.Ref{V: "$schema.hi"},
+				Schema: &schemahcl.Ref{V: "$schema.hi"},
 				Columns: []*sqlspec.Column{
 					{
 						Name:    "id",
-						Type:    &schemaspec.Type{T: "int"},
+						Type:    &schemahcl.Type{T: "int"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "123"},
+						Default: &schemahcl.LiteralValue{V: "123"},
 					},
 					{
 						Name:    "age",
-						Type:    &schemaspec.Type{T: "int"},
+						Type:    &schemahcl.Type{T: "int"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "10"},
+						Default: &schemahcl.LiteralValue{V: "10"},
 					},
 					{
 						Name:    "active",
-						Type:    &schemaspec.Type{T: "boolean"},
+						Type:    &schemahcl.Type{T: "boolean"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "true"},
+						Default: &schemahcl.LiteralValue{V: "true"},
 					},
 					{
 						Name:    "account_active",
-						Type:    &schemaspec.Type{T: "boolean"},
+						Type:    &schemahcl.Type{T: "boolean"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "true"},
+						Default: &schemahcl.LiteralValue{V: "true"},
 					},
 				},
 				PrimaryKey: &sqlspec.PrimaryKey{
-					Columns: []*schemaspec.Ref{
+					Columns: []*schemahcl.Ref{
 						{
 							V: "$table.users.$column.id",
 						},
@@ -181,7 +181,7 @@ table "accounts" {
 					{
 						Name:   "age",
 						Unique: true,
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.users.$column.age",
 							},
@@ -190,7 +190,7 @@ table "accounts" {
 					{
 						Name:   "active",
 						Unique: false,
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.users.$column.active",
 							},
@@ -200,51 +200,51 @@ table "accounts" {
 				ForeignKeys: []*sqlspec.ForeignKey{
 					{
 						Symbol: "fk",
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.users.$column.account_active",
 							},
 						},
-						RefColumns: []*schemaspec.Ref{
+						RefColumns: []*schemahcl.Ref{
 							{
 								V: "$table.accounts.$column.active",
 							},
 						},
-						OnDelete: &schemaspec.Ref{V: string(schema.SetNull)},
+						OnDelete: &schemahcl.Ref{V: string(schema.SetNull)},
 					},
 				},
 			},
 			{
 				Name:   "accounts",
-				Schema: &schemaspec.Ref{V: "$schema.hi"},
+				Schema: &schemahcl.Ref{V: "$schema.hi"},
 				Columns: []*sqlspec.Column{
 					{
 						Name:    "id",
-						Type:    &schemaspec.Type{T: "int"},
+						Type:    &schemahcl.Type{T: "int"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "123"},
+						Default: &schemahcl.LiteralValue{V: "123"},
 					},
 					{
 						Name:    "age",
-						Type:    &schemaspec.Type{T: "int"},
+						Type:    &schemahcl.Type{T: "int"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "10"},
+						Default: &schemahcl.LiteralValue{V: "10"},
 					},
 					{
 						Name:    "active",
-						Type:    &schemaspec.Type{T: "boolean"},
+						Type:    &schemahcl.Type{T: "boolean"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "true"},
+						Default: &schemahcl.LiteralValue{V: "true"},
 					},
 					{
 						Name:    "user_active",
-						Type:    &schemaspec.Type{T: "boolean"},
+						Type:    &schemahcl.Type{T: "boolean"},
 						Null:    false,
-						Default: &schemaspec.LiteralValue{V: "true"},
+						Default: &schemahcl.LiteralValue{V: "true"},
 					},
 				},
 				PrimaryKey: &sqlspec.PrimaryKey{
-					Columns: []*schemaspec.Ref{
+					Columns: []*schemahcl.Ref{
 						{
 							V: "$table.accounts.$column.id",
 						},
@@ -254,7 +254,7 @@ table "accounts" {
 					{
 						Name:   "age",
 						Unique: true,
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.accounts.$column.age",
 							},
@@ -263,7 +263,7 @@ table "accounts" {
 					{
 						Name:   "active",
 						Unique: false,
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.accounts.$column.active",
 							},
@@ -273,17 +273,17 @@ table "accounts" {
 				ForeignKeys: []*sqlspec.ForeignKey{
 					{
 						Symbol: "fk",
-						Columns: []*schemaspec.Ref{
+						Columns: []*schemahcl.Ref{
 							{
 								V: "$table.accounts.$column.user_active",
 							},
 						},
-						RefColumns: []*schemaspec.Ref{
+						RefColumns: []*schemahcl.Ref{
 							{
 								V: "$table.users.$column.active",
 							},
 						},
-						OnDelete: &schemaspec.Ref{V: string(schema.SetNull)},
+						OnDelete: &schemahcl.Ref{V: string(schema.SetNull)},
 					},
 				},
 			},
@@ -302,10 +302,10 @@ schema "hi" {
 		Schemas: []*sqlspec.Schema{
 			{
 				Name: "hi",
-				DefaultExtension: schemaspec.DefaultExtension{
-					Extra: schemaspec.Resource{
-						Attrs: []*schemaspec.Attr{
-							{K: "x", V: &schemaspec.LiteralValue{V: "1"}},
+				DefaultExtension: schemahcl.DefaultExtension{
+					Extra: schemahcl.Resource{
+						Attrs: []*schemahcl.Attr{
+							{K: "x", V: &schemahcl.LiteralValue{V: "1"}},
 						},
 					},
 				},
@@ -344,6 +344,8 @@ table "accounts" {
 	require.NoError(t, err)
 }
 
+var hcl = schemahcl.New(schemahcl.WithTypes(postgres.TypeRegistry.Specs()))
+
 func TestMarshalTopLevel(t *testing.T) {
 	c := &sqlspec.Schema{
 		Name: "schema",
@@ -371,7 +373,7 @@ table "t2" {
 	for _, tt := range dialects {
 		t.Run(tt.name, func(t *testing.T) {
 			var r schema.Realm
-			err := tt.UnmarshalSpec([]byte(f), &r)
+			err := tt.Eval([]byte(f), &r, nil)
 			require.NoError(t, err)
 			exp := &schema.Realm{
 				Schemas: []*schema.Schema{
@@ -397,7 +399,7 @@ table "t2" {
 			hcl, err := tt.MarshalSpec(&r)
 			require.NoError(t, err)
 			var after schema.Realm
-			err = tt.UnmarshalSpec(hcl, &after)
+			err = tt.Eval(hcl, &after, nil)
 			require.NoError(t, err)
 			require.EqualValues(t, exp, &after)
 		})
@@ -418,7 +420,7 @@ func TestUnsignedImmutability(t *testing.T) {
 schema "test" {
 }`
 	var s schema.Schema
-	err := mysql.UnmarshalHCL([]byte(f), &s)
+	err := mysql.EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 	tbl := s.Tables[0]
 	require.EqualValues(t, &schema.IntegerType{T: "bigint", Unsigned: true}, tbl.Columns[0].Type.Type)
@@ -459,7 +461,7 @@ table "b" "users" {
 }
 `
 	var r schema.Realm
-	err := mysql.UnmarshalHCL([]byte(h), &r)
+	err := mysql.EvalHCLBytes([]byte(h), &r, nil)
 	require.NoError(t, err)
 
 	require.EqualValues(t, r.Schemas[0].Tables[0].Columns[0], r.Schemas[1].Tables[0].ForeignKeys[0].RefColumns[0])
@@ -521,7 +523,7 @@ schema "c" {
 
 func decode(f string) (*db, error) {
 	d := &db{}
-	if err := hcl.UnmarshalSpec([]byte(f), d); err != nil {
+	if err := hcl.EvalBytes([]byte(f), d, nil); err != nil {
 		return nil, err
 	}
 	return d, nil
