@@ -9,6 +9,7 @@ import (
 	"context"
 	"database/sql"
 	"flag"
+	"fmt"
 	"io"
 	"io/ioutil"
 	"os"
@@ -20,33 +21,29 @@ import (
 	"text/template"
 	"time"
 
-	entmigrate "ariga.io/atlas/cmd/atlascmd/migrate"
-	"ariga.io/atlas/cmd/atlascmd/migrate/ent/revision"
-	"ariga.io/atlas/schema/schemaspec"
+	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
-	"ariga.io/atlas/sql/sqlclient"
 	entsql "entgo.io/ent/dialect/sql"
 	entschema "entgo.io/ent/dialect/sql/schema"
 	"entgo.io/ent/entc/integration/ent"
+	"github.com/hashicorp/hcl/v2/hclparse"
 	"github.com/stretchr/testify/require"
 )
 
+var (
+	dbs         []io.Closer
+	flagVersion string
+)
+
 func TestMain(m *testing.M) {
-	var dialect string
-	flag.StringVar(&dialect, "dialect", "", "[mysql56, postgres10, tidb5, ...] what dialect (version) to test")
+	flag.StringVar(&flagVersion, "version", "", "[mysql56, postgres10, tidb5, ...] what version to test")
 	flag.Parse()
-	var dbs []io.Closer
-	dbs = append(dbs, myInit(dialect)...)
-	dbs = append(dbs, pgInit(dialect)...)
-	dbs = append(dbs, tidbInit(dialect)...)
-	dbs = append(dbs, crdbInit(dialect)...)
-	defer func() {
-		for _, db := range dbs {
-			db.Close()
-		}
-	}()
-	os.Exit(m.Run())
+	code := m.Run()
+	for _, db := range dbs {
+		db.Close()
+	}
+	os.Exit(code)
 }
 
 // T holds the elements common between dialect tests.
@@ -179,12 +176,12 @@ func testHCLIntegration(t T, full string, empty string) {
 	require.Empty(t, t.realm().Schemas[0].Tables)
 }
 
-func testCLISchemaInspect(t T, h string, dsn string, unmarshaler schemaspec.Unmarshaler, args ...string) {
+func testCLISchemaInspect(t T, h string, dsn string, eval schemahcl.Evaluator, args ...string) {
 	err := initCLI()
 	require.NoError(t, err)
 	t.dropTables("users")
 	var expected schema.Schema
-	err = unmarshaler.UnmarshalSpec([]byte(h), &expected)
+	err = evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyHcl(h)
 	runArgs := []string{
@@ -201,18 +198,18 @@ func testCLISchemaInspect(t T, h string, dsn string, unmarshaler schemaspec.Unma
 	cmd.Stdout = stdout
 	require.NoError(t, cmd.Run(), stderr.String())
 	var actual schema.Schema
-	err = unmarshaler.UnmarshalSpec(stdout.Bytes(), &actual)
+	err = evalBytes(stdout.Bytes(), &actual, eval)
 	require.NoError(t, err)
 	require.Empty(t, stderr.String())
 	require.Equal(t, expected, actual)
 }
 
-func testCLISchemaInspectEnv(t T, h string, env string, unmarshaler schemaspec.Unmarshaler) {
+func testCLISchemaInspectEnv(t T, h string, env string, eval schemahcl.Evaluator) {
 	err := initCLI()
 	require.NoError(t, err)
 	t.dropTables("users")
 	var expected schema.Schema
-	err = unmarshaler.UnmarshalSpec([]byte(h), &expected)
+	err = evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyHcl(h)
 	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
@@ -226,7 +223,7 @@ func testCLISchemaInspectEnv(t T, h string, env string, unmarshaler schemaspec.U
 	cmd.Stdout = stdout
 	require.NoError(t, cmd.Run(), stderr.String())
 	var actual schema.Schema
-	err = unmarshaler.UnmarshalSpec(stdout.Bytes(), &actual)
+	err = evalBytes(stdout.Bytes(), &actual, eval)
 	require.NoError(t, err)
 	require.Empty(t, stderr.String())
 	require.Equal(t, expected, actual)
@@ -243,14 +240,14 @@ func initCLI() error {
 	return err
 }
 
-func testCLIMultiSchemaApply(t T, h string, dsn string, schemas []string, unmarshaler schemaspec.Unmarshaler) {
+func testCLIMultiSchemaApply(t T, h string, dsn string, schemas []string, eval schemahcl.Evaluator) {
 	err := initCLI()
-	f := filepath.Join(t.TempDir(), "atlas.hcl")
+	f := filepath.Join(t.TempDir(), "schema.hcl")
 	err = ioutil.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	require.NoError(t, err)
 	var expected schema.Realm
-	err = unmarshaler.UnmarshalSpec([]byte(h), &expected)
+	err = evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
 		"schema",
@@ -273,11 +270,11 @@ func testCLIMultiSchemaApply(t T, h string, dsn string, schemas []string, unmars
 	require.Contains(t, stdout.String(), `-- Add new schema named "test2"`)
 }
 
-func testCLIMultiSchemaInspect(t T, h string, dsn string, schemas []string, unmarshaler schemaspec.Unmarshaler) {
+func testCLIMultiSchemaInspect(t T, h string, dsn string, schemas []string, eval schemahcl.Evaluator) {
 	err := initCLI()
 	require.NoError(t, err)
 	var expected schema.Realm
-	err = unmarshaler.UnmarshalSpec([]byte(h), &expected)
+	err = evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyRealmHcl(h)
 	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
@@ -293,7 +290,7 @@ func testCLIMultiSchemaInspect(t T, h string, dsn string, schemas []string, unma
 	cmd.Stdout = stdout
 	require.NoError(t, cmd.Run(), stderr.String())
 	var actual schema.Realm
-	err = unmarshaler.UnmarshalSpec(stdout.Bytes(), &actual)
+	err = evalBytes(stdout.Bytes(), &actual, eval)
 	require.NoError(t, err)
 	require.Empty(t, stderr.String())
 	require.Equal(t, expected, actual)
@@ -303,7 +300,7 @@ func testCLISchemaApply(t T, h string, dsn string, args ...string) {
 	err := initCLI()
 	require.NoError(t, err)
 	t.dropTables("users")
-	f := filepath.Join(t.TempDir(), "atlas.hcl")
+	f := filepath.Join(t.TempDir(), "schema.hcl")
 	err = ioutil.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	runArgs := []string{
@@ -338,7 +335,7 @@ func testCLISchemaApplyDry(t T, h string, dsn string) {
 	err := initCLI()
 	require.NoError(t, err)
 	t.dropTables("users")
-	f := filepath.Join(t.TempDir(), "atlas.hcl")
+	f := filepath.Join(t.TempDir(), "schema.hcl")
 	err = ioutil.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
@@ -371,7 +368,7 @@ func testCLISchemaApplyAutoApprove(t T, h string, dsn string, args ...string) {
 	err := initCLI()
 	require.NoError(t, err)
 	t.dropTables("users")
-	f := filepath.Join(t.TempDir(), "atlas.hcl")
+	f := filepath.Join(t.TempDir(), "schema.hcl")
 	err = ioutil.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	runArgs := []string{
@@ -472,25 +469,6 @@ func testAdvisoryLock(t *testing.T, l schema.Locker) {
 	})
 }
 
-// testEntRevisions will not be run for SQLite. It has its own version of this test.
-func testEntRevisions(t T) {
-	const name = "my_revisions_schema"
-
-	c, err := sqlclient.Open(context.Background(), t.url())
-	require.NoError(t, err)
-
-	r, err := entmigrate.NewEntRevisions(c, entmigrate.WithSchema(name))
-	require.NoError(t, err)
-
-	require.NoError(t, r.Init(context.Background()))
-	t.dropSchemas(name)
-
-	s, err := c.InspectSchema(context.Background(), name, nil)
-	require.NoError(t, err)
-	_, ok := s.Table(revision.Table)
-	require.True(t, ok)
-}
-
 func testExecutor(t T) {
 	usersT, postsT := t.users(), t.posts()
 	petsT := &schema.Table{
@@ -547,6 +525,10 @@ func plan(t T, name string, changes ...schema.Change) *migrate.Plan {
 
 type rrw migrate.Revisions
 
+func (r *rrw) Ident() *migrate.TableIdent {
+	return &migrate.TableIdent{}
+}
+
 func (r *rrw) WriteRevision(_ context.Context, rev *migrate.Revision) error {
 	for i, rev2 := range *r {
 		if rev2.Version == rev.Version {
@@ -558,6 +540,15 @@ func (r *rrw) WriteRevision(_ context.Context, rev *migrate.Revision) error {
 	return nil
 }
 
+func (r *rrw) ReadRevision(_ context.Context, v string) (*migrate.Revision, error) {
+	for _, rev := range *r {
+		if rev.Version == v {
+			return rev, nil
+		}
+	}
+	return nil, migrate.ErrRevisionNotExist
+}
+
 func (r *rrw) ReadRevisions(context.Context) (migrate.Revisions, error) {
 	return migrate.Revisions(*r), nil
 }
@@ -568,8 +559,16 @@ func (r *rrw) clean() {
 
 func buildCmd(t *testing.T) (string, error) {
 	td := t.TempDir()
-	if err := exec.Command("go", "build", "-o", td, "ariga.io/atlas/cmd/atlas").Run(); err != nil {
-		return "", err
+	if b, err := exec.Command("go", "build", "-o", td, "ariga.io/atlas/cmd/atlas").CombinedOutput(); err != nil {
+		return "", fmt.Errorf("%w: %s", err, b)
 	}
 	return filepath.Join(td, "atlas"), nil
 }
+
+func evalBytes(b []byte, v interface{}, ev schemahcl.Evaluator) error {
+	p := hclparse.NewParser()
+	if _, diag := p.ParseHCL(b, ""); diag.HasErrors() {
+		return diag
+	}
+	return ev.Eval(p, v, nil)
+}
