@@ -8,9 +8,9 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
-	"io"
 	"log"
 	"strings"
+	"sync"
 	"testing"
 
 	"ariga.io/atlas/sql/migrate"
@@ -28,63 +28,43 @@ type myTest struct {
 	rrw     migrate.RevisionReadWriter
 	version string
 	port    int
+	once    sync.Once
 }
 
-var myTests = struct {
-	drivers map[string]*myTest
-	ports   map[string]int
-}{
-	drivers: make(map[string]*myTest),
-	ports: map[string]int{
-		"mysql56":  3306,
-		"mysql57":  3307,
-		"mysql8":   3308,
-		"maria107": 4306,
-		"maria102": 4307,
-		"maria103": 4308,
-	},
-}
-
-func myInit(d string) []io.Closer {
-	var cs []io.Closer
-	if d != "" {
-		p, ok := myTests.ports[d]
-		if ok {
-			myTests.ports = map[string]int{d: p}
-		} else {
-			myTests.ports = make(map[string]int)
-		}
-	}
-	for version, port := range myTests.ports {
-		db, err := sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/test?parseTime=True", port))
-		if err != nil {
-			log.Fatalln(err)
-		}
-		cs = append(cs, db)
-		drv, err := mysql.Open(db)
-		if err != nil {
-			log.Fatalln(err)
-		}
-		myTests.drivers[version] = &myTest{db: db, drv: drv, version: version, port: port, rrw: &rrw{}}
-	}
-	return cs
+var myTests = map[string]*myTest{
+	"mysql56":  {port: 3306},
+	"mysql57":  {port: 3307},
+	"mysql8":   {port: 3308},
+	"maria107": {port: 4306},
+	"maria102": {port: 4307},
+	"maria103": {port: 4308},
 }
 
 func myRun(t *testing.T, fn func(*myTest)) {
-	for version, tt := range myTests.drivers {
-		t.Run(version, func(t *testing.T) {
-			tt := &myTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
-			fn(tt)
-		})
+	for version, tt := range myTests {
+		if flagVersion == "" || flagVersion == version {
+			t.Run(version, func(t *testing.T) {
+				tt.once.Do(func() {
+					var err error
+					tt.version = version
+					tt.rrw = &rrw{}
+					tt.db, err = sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/test?parseTime=True", tt.port))
+					if err != nil {
+						log.Fatalln(err)
+					}
+					dbs = append(dbs, tt.db) // close connection after all tests have been run
+					tt.drv, err = mysql.Open(tt.db)
+					if err != nil {
+						log.Fatalln(err)
+					}
+				})
+				tt := &myTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
+				fn(tt)
+			})
+		}
 	}
 }
 
-func TestMySQL_EntRevisions(t *testing.T) {
-	myRun(t, func(t *myTest) {
-		testEntRevisions(t)
-	})
-}
-
 func TestMySQL_Executor(t *testing.T) {
 	myRun(t, func(t *myTest) {
 		testExecutor(t)
@@ -568,6 +548,39 @@ schema "test" {
 	})
 }
 
+func TestMySQL_Snapshot(t *testing.T) {
+	myRun(t, func(t *myTest) {
+		db, err := sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/", t.port))
+		require.NoError(t, err)
+		t.Cleanup(func() {
+			require.NoError(t, db.Close())
+		})
+		drv, err := mysql.Open(db)
+		require.NoError(t, err)
+
+		_, err = drv.(migrate.Snapshoter).Snapshot(context.Background())
+		require.ErrorAs(t, err, &migrate.NotCleanError{})
+
+		r, err := t.driver().InspectRealm(context.Background(), nil)
+		require.NoError(t, err)
+		restore, err := t.driver().(migrate.Snapshoter).Snapshot(context.Background())
+		require.NoError(t, err) // connected to test schema
+		t.migrate(&schema.AddTable{T: schema.NewTable("my_table").AddColumns(
+			schema.NewIntColumn("col_1", "integer").SetNull(true),
+			schema.NewIntColumn("col_2", "bigint"),
+		)})
+		t.Cleanup(func() {
+			t.dropTables("my_table")
+		})
+		require.NoError(t, restore(context.Background()))
+		r1, err := t.driver().InspectRealm(context.Background(), nil)
+		require.NoError(t, err)
+		diff, err := t.driver().RealmDiff(r1, r)
+		require.NoError(t, err)
+		require.Zero(t, diff)
+	})
+}
+
 func TestMySQL_CLI(t *testing.T) {
 	h := `
 			schema "test" {
@@ -587,7 +600,7 @@ func TestMySQL_CLI(t *testing.T) {
 		myRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"), mysql.UnmarshalHCL)
+			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"), mysql.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
@@ -671,7 +684,7 @@ func TestMySQL_CLI_MultiSchema(t *testing.T) {
 			t.dropTables("users")
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLIMultiSchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.UnmarshalHCL)
+			testCLIMultiSchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
@@ -680,7 +693,7 @@ func TestMySQL_CLI_MultiSchema(t *testing.T) {
 			t.dropTables("users")
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLIMultiSchemaApply(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.UnmarshalHCL)
+			testCLIMultiSchemaApply(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.EvalHCL)
 		})
 	})
 }
@@ -705,6 +718,67 @@ schema "second" {
 	})
 }
 
+func TestMySQL_HCL_ForeignKeyCrossSchema(t *testing.T) {
+	const expected = `table "credit_cards" {
+  schema = schema.financial
+  column "id" {
+    null = false
+    type = int
+  }
+  column "user_id" {
+    null = false
+    type = int
+  }
+  primary_key {
+    columns = [column.id]
+  }
+  foreign_key "user_id_fkey" {
+    columns     = [column.user_id]
+    ref_columns = [table.users.users.column.id]
+    on_update   = NO_ACTION
+    on_delete   = NO_ACTION
+  }
+  index "user_id_fkey" {
+    columns = [column.user_id]
+  }
+}
+table "users" "users" {
+  schema = schema.users
+  column "id" {
+    null = false
+    type = int
+  }
+  column "email" {
+    null = false
+    type = varchar(255)
+  }
+  primary_key {
+    columns = [column.id]
+  }
+}
+schema "financial" {
+  charset = "utf8mb4"
+  collate = "utf8mb4_general_ci"
+}
+schema "users" {
+  charset = "utf8mb4"
+  collate = "utf8mb4_general_ci"
+}
+`
+	myRun(t, func(t *myTest) {
+		t.dropSchemas("financial", "users")
+		realm := t.loadRealm()
+		hcl, err := mysql.MarshalHCL(realm)
+		require.NoError(t, err)
+		t.applyRealmHcl(string(hcl) + "\n" + expected)
+		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{Schemas: []string{"users", "financial"}})
+		require.NoError(t, err)
+		actual, err := mysql.MarshalHCL(realm)
+		require.NoError(t, err)
+		require.Equal(t, expected, string(actual))
+	})
+}
+
 func TestMySQL_DefaultsHCL(t *testing.T) {
 	n := "atlas_defaults"
 	myRun(t, func(t *myTest) {
@@ -725,7 +799,7 @@ create table atlas_defaults
 		spec, err := mysql.MarshalHCL(realm.Schemas[0])
 		require.NoError(t, err)
 		var s schema.Realm
-		err = mysql.UnmarshalHCL(spec, &s)
+		err = mysql.EvalHCLBytes(spec, &s, nil)
 		require.NoError(t, err)
 		t.dropTables(n)
 		t.applyHcl(string(spec))
@@ -800,7 +874,7 @@ create table atlas_types_sanity
 				Columns: []*schema.Column{
 					{
 						Name:    "tBit",
-						Type:    &schema.ColumnType{Type: &mysql.BitType{T: "bit"}, Raw: "bit(10)", Null: true},
+						Type:    &schema.ColumnType{Type: &mysql.BitType{T: "bit", Size: 10}, Raw: "bit(10)", Null: true},
 						Default: &schema.Literal{V: "b'100'"},
 					},
 					{
@@ -987,34 +1061,34 @@ create table atlas_types_sanity
 					},
 					{
 						Name: "tVarBinary",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "varbinary", Size: 30},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "varbinary", Size: intp(30)},
 							Raw: "varbinary(30)", Null: true},
 						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"mysql8": "0x546974616E"}, t.quoted("Titan"))},
 					},
 					{
 						Name: "tBinary",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "binary", Size: 5},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "binary", Size: intp(5)},
 							Raw: "binary(5)", Null: true},
 						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"mysql8": "0x546974616E"}, t.quoted("Titan"))},
 					},
 					{
 						Name: "tBlob",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "tinyblob", Size: 0},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "tinyblob"},
 							Raw: "tinyblob", Null: true},
 					},
 					{
 						Name: "tTinyBlob",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "tinyblob", Size: 0},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "tinyblob"},
 							Raw: "tinyblob", Null: true},
 					},
 					{
 						Name: "tMediumBlob",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "mediumblob", Size: 0},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "mediumblob"},
 							Raw: "mediumblob", Null: true},
 					},
 					{
 						Name: "tLongBlob",
-						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "longblob", Size: 0},
+						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "longblob"},
 							Raw: "longblob", Null: true},
 					},
 					{
@@ -1211,7 +1285,7 @@ func (t *myTest) revisionsStorage() migrate.RevisionReadWriter {
 func (t *myTest) applyHcl(spec string) {
 	realm := t.loadRealm()
 	var desired schema.Schema
-	err := mysql.UnmarshalHCL([]byte(spec), &desired)
+	err := mysql.EvalHCLBytes([]byte(spec), &desired, nil)
 	require.NoError(t, err)
 	existing := realm.Schemas[0]
 	require.NoError(t, err)
@@ -1224,7 +1298,7 @@ func (t *myTest) applyHcl(spec string) {
 func (t *myTest) applyRealmHcl(spec string) {
 	realm := t.loadRealm()
 	var desired schema.Realm
-	err := mysql.UnmarshalHCL([]byte(spec), &desired)
+	err := mysql.EvalHCLBytes([]byte(spec), &desired, nil)
 	require.NoError(t, err)
 	diff, err := t.drv.RealmDiff(realm, &desired)
 	require.NoError(t, err)
