@@ -8,15 +8,18 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
-	"io"
 	"log"
+	"os"
+	"os/exec"
+	"path/filepath"
 	"strings"
+	"sync"
 	"testing"
 
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
-
+	"ariga.io/atlas/sql/sqlclient"
 	"entgo.io/ent/dialect"
 	_ "github.com/lib/pq"
 	"github.com/stretchr/testify/require"
@@ -29,62 +32,42 @@ type pgTest struct {
 	rrw     migrate.RevisionReadWriter
 	version string
 	port    int
+	once    sync.Once
 }
 
-var pgTests = struct {
-	drivers map[string]*pgTest
-	ports   map[string]int
-}{
-	drivers: make(map[string]*pgTest),
-	ports: map[string]int{
-		"postgres10": 5430,
-		"postgres11": 5431,
-		"postgres12": 5432,
-		"postgres13": 5433,
-		"postgres14": 5434,
-	},
-}
-
-func pgInit(d string) []io.Closer {
-	var cs []io.Closer
-	if d != "" {
-		p, ok := pgTests.ports[d]
-		if ok {
-			pgTests.ports = map[string]int{d: p}
-		} else {
-			pgTests.ports = make(map[string]int)
-		}
-	}
-	for version, port := range pgTests.ports {
-		db, err := sql.Open("postgres", fmt.Sprintf("host=localhost port=%d user=postgres dbname=test password=pass sslmode=disable", port))
-		if err != nil {
-			log.Fatalln(err)
-		}
-		cs = append(cs, db)
-		drv, err := postgres.Open(db)
-		if err != nil {
-			log.Fatalln(err)
-		}
-		pgTests.drivers[version] = &pgTest{db: db, drv: drv, version: version, port: port, rrw: &rrw{}}
-	}
-	return cs
+var pgTests = map[string]*pgTest{
+	"postgres10": {port: 5430},
+	"postgres11": {port: 5431},
+	"postgres12": {port: 5432},
+	"postgres13": {port: 5433},
+	"postgres14": {port: 5434},
 }
 
 func pgRun(t *testing.T, fn func(*pgTest)) {
-	for version, tt := range pgTests.drivers {
-		t.Run(version, func(t *testing.T) {
-			tt := &pgTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
-			fn(tt)
-		})
+	for version, tt := range pgTests {
+		if flagVersion == "" || flagVersion == version {
+			t.Run(version, func(t *testing.T) {
+				tt.once.Do(func() {
+					var err error
+					tt.version = version
+					tt.rrw = &rrw{}
+					tt.db, err = sql.Open("postgres", fmt.Sprintf("host=localhost port=%d user=postgres dbname=test password=pass sslmode=disable", tt.port))
+					if err != nil {
+						log.Fatalln(err)
+					}
+					dbs = append(dbs, tt.db) // close connection after all tests have been run
+					tt.drv, err = postgres.Open(tt.db)
+					if err != nil {
+						log.Fatalln(err)
+					}
+				})
+				tt := &pgTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
+				fn(tt)
+			})
+		}
 	}
 }
 
-func TestPostgres_EntRevisions(t *testing.T) {
-	pgRun(t, func(t *pgTest) {
-		testEntRevisions(t)
-	})
-}
-
 func TestPostgres_Executor(t *testing.T) {
 	pgRun(t, func(t *pgTest) {
 		testExecutor(t)
@@ -202,7 +185,7 @@ func TestPostgres_AddColumns(t *testing.T) {
 			&schema.Column{Name: "n", Type: &schema.ColumnType{Type: &schema.SpatialType{T: "point"}, Null: true}, Default: &schema.Literal{V: "'(1,2)'"}},
 			&schema.Column{Name: "o", Type: &schema.ColumnType{Type: &schema.SpatialType{T: "line"}, Null: true}, Default: &schema.Literal{V: "'{1,2,3}'"}},
 			&schema.Column{Name: "p", Type: &schema.ColumnType{Type: &postgres.UserDefinedType{T: "hstore"}, Null: true}, Default: &schema.RawExpr{X: "'a => 1'"}},
-			&schema.Column{Name: "q", Type: &schema.ColumnType{Type: &postgres.ArrayType{T: "text[]"}, Null: true}, Default: &schema.Literal{V: "'{}'"}},
+			&schema.Column{Name: "q", Type: &schema.ColumnType{Type: &postgres.ArrayType{Type: &schema.StringType{T: "text"}, T: "text[]"}, Null: true}, Default: &schema.Literal{V: "'{}'"}},
 		)
 		changes := t.diff(t.loadUsers(), usersT)
 		require.Len(t, changes, 17)
@@ -258,7 +241,7 @@ func TestPostgres_ColumnArray(t *testing.T) {
 		// Add column.
 		usersT.Columns = append(
 			usersT.Columns,
-			&schema.Column{Name: "a", Type: &schema.ColumnType{Raw: "int[]", Type: &postgres.ArrayType{T: "int[]"}}, Default: &schema.Literal{V: "'{1}'"}},
+			&schema.Column{Name: "a", Type: &schema.ColumnType{Raw: "int[]", Type: &postgres.ArrayType{Type: &schema.IntegerType{T: "int"}, T: "int[]"}}, Default: &schema.Literal{V: "'{1}'"}},
 		)
 		changes := t.diff(t.loadUsers(), usersT)
 		require.Len(t, changes, 1)
@@ -434,6 +417,51 @@ func TestPostgres_Ent(t *testing.T) {
 	pgRun(t, func(t *pgTest) {
 		testEntIntegration(t, dialect.Postgres, t.db)
 	})
+	// Migration to global unique identifiers.
+	t.Run("GlobalUniqueID", func(t *testing.T) {
+		pgRun(t, func(t *pgTest) {
+			ctx := context.Background()
+			t.dropTables("global_id")
+			_, err := t.driver().ExecContext(ctx, "CREATE TABLE global_id (id int NOT NULL GENERATED BY DEFAULT AS IDENTITY, PRIMARY KEY(id))")
+			require.NoError(t, err)
+			_, err = t.driver().ExecContext(ctx, "ALTER TABLE global_id ALTER COLUMN id RESTART WITH 1024")
+			require.NoError(t, err)
+			_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
+			require.NoError(t, err)
+			var id int
+			require.NoError(t, t.db.QueryRow("SELECT id FROM global_id").Scan(&id))
+			require.Equal(t, 1024, id)
+			_, err = t.driver().ExecContext(ctx, "DELETE FROM global_id WHERE id = 1024")
+			require.NoError(t, err)
+
+			globalT := t.loadTable("global_id")
+			c, ok := globalT.Column("id")
+			require.True(t, ok)
+			require.EqualValues(t, 1, c.Attrs[0].(*postgres.Identity).Sequence.Start)
+			t.migrate(&schema.ModifyTable{
+				T: globalT,
+				Changes: []schema.Change{
+					&schema.ModifyColumn{
+						From: globalT.Columns[0],
+						To: schema.NewIntColumn("id", "int").
+							AddAttrs(&postgres.Identity{
+								Generation: "BY DEFAULT",
+								Sequence: &postgres.Sequence{
+									Start: 1024,
+								},
+							}),
+						Change: schema.ChangeAttr,
+					},
+				},
+			})
+			_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
+			require.NoError(t, err)
+			globalT = t.loadTable("global_id")
+			c, ok = globalT.Column("id")
+			require.True(t, ok)
+			require.EqualValues(t, 1024, c.Attrs[0].(*postgres.Identity).Sequence.Start)
+		})
+	})
 }
 
 func TestPostgres_AdvisoryLock(t *testing.T) {
@@ -508,10 +536,64 @@ schema "second" {
 	})
 }
 
+func TestPostgres_HCL_ForeignKeyCrossSchema(t *testing.T) {
+	const expected = `table "credit_cards" {
+  schema = schema.financial
+  column "id" {
+    null = false
+    type = serial
+  }
+  column "user_id" {
+    null = false
+    type = integer
+  }
+  primary_key {
+    columns = [column.id]
+  }
+  foreign_key "user_id_fkey" {
+    columns     = [column.user_id]
+    ref_columns = [table.users.users.column.id]
+    on_update   = NO_ACTION
+    on_delete   = NO_ACTION
+  }
+}
+table "users" "users" {
+  schema = schema.users
+  column "id" {
+    null = false
+    type = bigserial
+  }
+  column "email" {
+    null = false
+    type = character_varying
+  }
+  primary_key {
+    columns = [column.id]
+  }
+}
+schema "financial" {
+}
+schema "users" {
+}
+`
+	pgRun(t, func(t *pgTest) {
+		t.dropSchemas("financial", "users")
+		realm := t.loadRealm()
+		hcl, err := postgres.MarshalHCL(realm)
+		require.NoError(t, err)
+		t.applyRealmHcl(string(hcl) + "\n" + expected)
+		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{Schemas: []string{"users", "financial"}})
+		require.NoError(t, err)
+		actual, err := postgres.MarshalHCL(realm)
+		require.NoError(t, err)
+		require.Equal(t, expected, string(actual))
+	})
+}
+
 func (t *pgTest) applyRealmHcl(spec string) {
 	realm := t.loadRealm()
 	var desired schema.Realm
-	err := postgres.UnmarshalHCL([]byte(spec), &desired)
+	err := postgres.EvalHCLBytes([]byte(spec), &desired, nil)
 	require.NoError(t, err)
 	diff, err := t.drv.RealmDiff(realm, &desired)
 	require.NoError(t, err)
@@ -519,6 +601,46 @@ func (t *pgTest) applyRealmHcl(spec string) {
 	require.NoError(t, err)
 }
 
+func TestPostgres_Snapshot(t *testing.T) {
+	pgRun(t, func(t *pgTest) {
+		client, err := sqlclient.Open(context.Background(), fmt.Sprintf("postgres://postgres:pass@localhost:%d/test?sslmode=disable&search_path=another", t.port))
+		require.NoError(t, err)
+
+		_, err = client.ExecContext(context.Background(), "CREATE SCHEMA another")
+		require.NoError(t, err)
+		t.Cleanup(func() {
+			_, err = client.ExecContext(context.Background(), "DROP SCHEMA IF EXISTS another")
+			require.NoError(t, client.Close())
+		})
+		drv := client.Driver
+
+		_, err = t.driver().(migrate.Snapshoter).Snapshot(context.Background())
+		require.ErrorAs(t, err, &migrate.NotCleanError{})
+
+		r, err := drv.InspectRealm(context.Background(), nil)
+		require.NoError(t, err)
+		restore, err := drv.(migrate.Snapshoter).Snapshot(context.Background())
+		require.NoError(t, err) // connected to test schema
+		require.NoError(t, drv.ApplyChanges(context.Background(), []schema.Change{
+			&schema.AddTable{T: schema.NewTable("my_table").
+				AddColumns(
+					schema.NewIntColumn("col_1", "integer").SetNull(true),
+					schema.NewIntColumn("col_2", "bigint"),
+				),
+			},
+		}))
+		t.Cleanup(func() {
+			t.dropTables("my_table")
+		})
+		require.NoError(t, restore(context.Background()))
+		r1, err := drv.InspectRealm(context.Background(), nil)
+		require.NoError(t, err)
+		diff, err := drv.RealmDiff(r1, r)
+		require.NoError(t, err)
+		require.Zero(t, diff)
+	})
+}
+
 func TestPostgres_CLI(t *testing.T) {
 	h := `
 			schema "public" {
@@ -534,17 +656,17 @@ func TestPostgres_CLI(t *testing.T) {
 			}`
 	t.Run("SchemaInspect", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaInspect(t, h, t.dsn(), postgres.UnmarshalHCL)
+			testCLISchemaInspect(t, h, t.dsn(""), postgres.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaApply(t, h, t.dsn())
+			testCLISchemaApply(t, h, t.dsn(""))
 		})
 	})
 	t.Run("SchemaApplyDryRun", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaApplyDry(t, h, t.dsn())
+			testCLISchemaApplyDry(t, h, t.dsn(""))
 		})
 	})
 	t.Run("SchemaApplyWithVars", func(t *testing.T) {
@@ -563,17 +685,17 @@ table "users" {
 }
 `
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaApply(t, h, t.dsn(), "--var", "tenant=public")
+			testCLISchemaApply(t, h, t.dsn(""), "--var", "tenant=public")
 		})
 	})
 	t.Run("SchemaDiffRun", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaDiff(t, t.dsn())
+			testCLISchemaDiff(t, t.dsn(""))
 		})
 	})
 	t.Run("SchemaApplyAutoApprove", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
-			testCLISchemaApplyAutoApprove(t, h, t.dsn())
+			testCLISchemaApplyAutoApprove(t, h, t.dsn(""))
 		})
 	})
 }
@@ -606,18 +728,90 @@ func TestPostgres_CLI_MultiSchema(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
 			t.dropSchemas("test2")
 			t.dropTables("users")
-			testCLIMultiSchemaInspect(t, h, t.dsn(), []string{"public", "test2"}, postgres.UnmarshalHCL)
+			testCLIMultiSchemaInspect(t, h, t.dsn(""), []string{"public", "test2"}, postgres.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		pgRun(t, func(t *pgTest) {
 			t.dropSchemas("test2")
 			t.dropTables("users")
-			testCLIMultiSchemaApply(t, h, t.dsn(), []string{"public", "test2"}, postgres.UnmarshalHCL)
+			testCLIMultiSchemaApply(t, h, t.dsn(""), []string{"public", "test2"}, postgres.EvalHCL)
 		})
 	})
 }
 
+func TestPostgres_MigrateDiffRealm(t *testing.T) {
+	bin, err := buildCmd(t)
+	require.NoError(t, err)
+	pgRun(t, func(t *pgTest) {
+		dir := t.TempDir()
+		_, err := t.db.Exec("CREATE DATABASE migrate_diff")
+		require.NoError(t, err)
+		defer t.db.Exec("DROP DATABASE IF EXISTS migrate_diff")
+
+		hcl := `
+schema "public" {}
+table "users" {
+	schema = schema.public
+	column "id" { type = integer }
+}
+schema "other" {}
+table "posts" {
+	schema = schema.other
+	column "id" { type = integer }
+}
+`
+		err = os.WriteFile(filepath.Join(dir, "schema.hcl"), []byte(hcl), 0600)
+		diff := func(name string) string {
+			out, err := exec.Command(
+				bin, "migrate", "diff", name,
+				"--dir", fmt.Sprintf("file://%s", filepath.Join(dir, "migrations")),
+				"--to", fmt.Sprintf("file://%s", filepath.Join(dir, "schema.hcl")),
+				"--dev-url", fmt.Sprintf("postgres://postgres:pass@localhost:%d/migrate_diff?sslmode=disable", t.port),
+			).CombinedOutput()
+			require.NoError(t, err, string(out))
+			return strings.TrimSpace(string(out))
+		}
+		require.Empty(t, diff("initial"))
+
+		// Expect one file and read its contents.
+		files, err := os.ReadDir(filepath.Join(dir, "migrations"))
+		require.NoError(t, err)
+		require.Equal(t, 2, len(files))
+		require.Equal(t, "atlas.sum", files[1].Name())
+		b, err := os.ReadFile(filepath.Join(dir, "migrations", files[0].Name()))
+		require.NoError(t, err)
+		require.Equal(t,
+			`-- Add new schema named "other"
+CREATE SCHEMA "other";
+-- create "users" table
+CREATE TABLE "public"."users" ("id" integer NOT NULL);
+-- create "posts" table
+CREATE TABLE "other"."posts" ("id" integer NOT NULL);
+`, string(b))
+		require.Equal(t, "The migration directory is synced with the desired state, no changes to be made", diff("no_change"))
+
+		// Append a change to the schema and expect a migration to be created.
+		hcl += `
+table "other" "users" {
+	schema = schema.other
+	column "id" { type = integer }
+}`
+		err = os.WriteFile(filepath.Join(dir, "schema.hcl"), []byte(hcl), 0600)
+		require.Empty(t, diff("second"))
+		require.Equal(t, "The migration directory is synced with the desired state, no changes to be made", diff("no_change"))
+		files, err = os.ReadDir(filepath.Join(dir, "migrations"))
+		require.NoError(t, err)
+		require.Equal(t, 3, len(files), dir)
+		b, err = os.ReadFile(filepath.Join(dir, "migrations", files[1].Name()))
+		require.NoError(t, err)
+		require.Equal(t,
+			`-- create "users" table
+CREATE TABLE "other"."users" ("id" integer NOT NULL);
+`, string(b))
+	})
+}
+
 func TestPostgres_DefaultsHCL(t *testing.T) {
 	n := "atlas_defaults"
 	pgRun(t, func(t *pgTest) {
@@ -639,7 +833,7 @@ create table atlas_defaults
 		spec, err := postgres.MarshalHCL(realm.Schemas[0])
 		require.NoError(t, err)
 		var s schema.Schema
-		err = postgres.UnmarshalHCL(spec, &s)
+		err = postgres.EvalHCLBytes(spec, &s, nil)
 		require.NoError(t, err)
 		t.dropTables(n)
 		t.applyHcl(string(spec))
@@ -752,12 +946,12 @@ create table atlas_types_sanity
 				{
 					Name:    "tCharacter",
 					Type:    &schema.ColumnType{Type: &schema.StringType{T: "character", Size: 10}, Raw: "character", Null: true},
-					Default: &schema.RawExpr{X: "'atlas'::bpchar"},
+					Default: &schema.Literal{V: "'atlas'"},
 				},
 				{
 					Name:    "tChar",
 					Type:    &schema.ColumnType{Type: &schema.StringType{T: "character", Size: 10}, Raw: "character", Null: true},
-					Default: &schema.RawExpr{X: "'atlas'::bpchar"},
+					Default: &schema.Literal{V: "'atlas'"},
 				},
 				{
 					Name:    "tCharVar",
@@ -972,7 +1166,7 @@ create table atlas_types_sanity
 				},
 				{
 					Name: "tArray",
-					Type: &schema.ColumnType{Type: &postgres.ArrayType{T: "text[]"}, Raw: "ARRAY", Null: true},
+					Type: &schema.ColumnType{Type: &postgres.ArrayType{Type: &schema.StringType{T: "text"}, T: "text[]"}, Raw: "ARRAY", Null: true},
 					Default: &schema.Literal{
 						V: "'{}'",
 					},
@@ -1014,7 +1208,7 @@ create table atlas_types_sanity
 				},
 				{
 					Name: "tInterval",
-					Type: &schema.ColumnType{Type: &schema.UnsupportedType{T: "interval"}, Raw: "interval", Null: true},
+					Type: &schema.ColumnType{Type: &postgres.IntervalType{T: "interval", Precision: intp(6)}, Raw: "interval", Null: true},
 					Default: &schema.RawExpr{
 						X: "'04:00:00'::interval",
 					},
@@ -1039,11 +1233,14 @@ create table atlas_types_sanity
 }
 
 func (t *pgTest) url() string {
-	return t.dsn()
+	return t.dsn("")
 }
 
-func (t *pgTest) dsn() string {
-	return fmt.Sprintf("postgres://postgres:pass@localhost:%d/test?sslmode=disable", t.port)
+func (t *pgTest) dsn(schema string) string {
+	if schema == "" {
+		schema = "public"
+	}
+	return fmt.Sprintf("postgres://postgres:pass@localhost:%d/test?sslmode=disable&search_path=%s", t.port, schema)
 }
 
 func (t *pgTest) driver() migrate.Driver {
@@ -1057,7 +1254,7 @@ func (t *pgTest) revisionsStorage() migrate.RevisionReadWriter {
 func (t *pgTest) applyHcl(spec string) {
 	realm := t.loadRealm()
 	var desired schema.Schema
-	err := postgres.UnmarshalHCL([]byte(spec), &desired)
+	err := postgres.EvalHCLBytes([]byte(spec), &desired, nil)
 	require.NoError(t, err)
 	existing := realm.Schemas[0]
 	diff, err := t.drv.SchemaDiff(existing, &desired)
