@@ -18,6 +18,7 @@ import (
 	"testing"
 	"unicode"
 
+	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
@@ -34,14 +35,16 @@ func TestMySQL_Script(t *testing.T) {
 			Dir:   "testdata/mysql",
 			Setup: t.setupScript,
 			Cmds: map[string]func(ts *testscript.TestScript, neg bool, args []string){
-				"only":    cmdOnly,
-				"apply":   t.cmdApply,
-				"exist":   t.cmdExist,
-				"synced":  t.cmdSynced,
-				"cmphcl":  t.cmdCmpHCL,
-				"cmpshow": t.cmdCmpShow,
-				"execsql": t.cmdExec,
-				"atlas":   t.cmdCLI,
+				"only":        cmdOnly,
+				"apply":       t.cmdApply,
+				"exist":       t.cmdExist,
+				"synced":      t.cmdSynced,
+				"cmphcl":      t.cmdCmpHCL,
+				"cmpshow":     t.cmdCmpShow,
+				"cmpmig":      t.cmdCmpMig,
+				"execsql":     t.cmdExec,
+				"atlas":       t.cmdCLI,
+				"clearSchema": t.clearSchema,
 			},
 		})
 	})
@@ -53,13 +56,16 @@ func TestPostgres_Script(t *testing.T) {
 			Dir:   "testdata/postgres",
 			Setup: t.setupScript,
 			Cmds: map[string]func(ts *testscript.TestScript, neg bool, args []string){
-				"only":    cmdOnly,
-				"apply":   t.cmdApply,
-				"exist":   t.cmdExist,
-				"synced":  t.cmdSynced,
-				"cmphcl":  t.cmdCmpHCL,
-				"cmpshow": t.cmdCmpShow,
-				"execsql": t.cmdExec,
+				"only":        cmdOnly,
+				"apply":       t.cmdApply,
+				"exist":       t.cmdExist,
+				"synced":      t.cmdSynced,
+				"cmphcl":      t.cmdCmpHCL,
+				"cmpshow":     t.cmdCmpShow,
+				"cmpmig":      t.cmdCmpMig,
+				"execsql":     t.cmdExec,
+				"atlas":       t.cmdCLI,
+				"clearSchema": t.clearSchema,
 			},
 		})
 	})
@@ -107,7 +113,7 @@ func replaceDBURL(env *testscript.Env, url string) error {
 
 func (t *pgTest) setupScript(env *testscript.Env) error {
 	env.Setenv("version", t.version)
-	u := strings.ReplaceAll(t.dsn(), "/test", "/")
+	u := strings.ReplaceAll(t.dsn(""), "/test", "/")
 	if err := replaceDBURL(env, u); err != nil {
 		return err
 	}
@@ -147,7 +153,10 @@ var (
 	keyDrv *sqlite.Driver
 )
 
-const cliPathKey = "cli"
+const (
+	atlasPathKey   = "cli.atlas"
+	atlasciPathKey = "cli.atlasci"
+)
 
 func (t *liteTest) setupScript(env *testscript.Env) error {
 	db, err := sql.Open("sqlite3", fmt.Sprintf("file:%s?cache=shared&_fk=1",
@@ -177,11 +186,11 @@ func (t *liteTest) setupScript(env *testscript.Env) error {
 }
 
 func setupCLITest(t *testing.T, env *testscript.Env) error {
-	cliPath, err := buildCmd(t)
+	path, err := buildCmd(t)
 	if err != nil {
 		return err
 	}
-	env.Setenv(cliPathKey, cliPath)
+	env.Setenv(atlasPathKey, path)
 	return nil
 }
 
@@ -220,10 +229,6 @@ func (t *myTest) cmdCmpShow(ts *testscript.TestScript, _ bool, args []string) {
 	})
 }
 
-func (t *myTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
-	cmdCLI(ts, neg, args, t.dsn("test"))
-}
-
 func (t *pgTest) cmdCmpShow(ts *testscript.TestScript, _ bool, args []string) {
 	cmdCmpShow(ts, args, func(schema, name string) (string, error) {
 		buf, err := exec.Command("docker", "ps", "-qa", "-f", fmt.Sprintf("publish=%d", t.port)).CombinedOutput()
@@ -270,37 +275,6 @@ func (t *liteTest) cmdCmpShow(ts *testscript.TestScript, _ bool, args []string)
 	})
 }
 
-// cmdCmpMig compares the nth migration file under migrations/ to the provided file.
-// Because migration are created with the execution timestamp, lexicographic order of
-// the files in the directory is used to access the file of interest.
-func (t *liteTest) cmdCmpMig(ts *testscript.TestScript, _ bool, args []string) {
-	if len(args) < 2 {
-		ts.Fatalf("invalid number of args to 'cmpmig': %d", len(args))
-	}
-	expected := ts.ReadFile(args[1])
-	dir, err := os.ReadDir(ts.MkAbs("migrations"))
-	ts.Check(err)
-	idx, err := strconv.Atoi(args[0])
-	ts.Check(err)
-	current := 0
-	for _, f := range dir {
-		if f.IsDir() || !strings.HasSuffix(f.Name(), ".sql") {
-			continue
-		}
-		if current == idx {
-			actual := ts.ReadFile(filepath.Join("migrations", f.Name()))
-			var sb strings.Builder
-			if strings.TrimSpace(actual) == strings.TrimSpace(expected) {
-				return
-			}
-			ts.Check(diff.Text(f.Name(), args[1], expected, actual, &sb))
-			ts.Fatalf(sb.String())
-		}
-		current++
-	}
-	ts.Fatalf("could not find the #%d migration", idx)
-}
-
 func cmdCmpShow(ts *testscript.TestScript, args []string, show func(schema, name string) (string, error)) {
 	if len(args) < 2 {
 		ts.Fatalf("invalid number of args to 'cmpshow': %d", len(args))
@@ -396,18 +370,26 @@ func (t *liteTest) cmdExec(ts *testscript.TestScript, _ bool, args []string) {
 	cmdExec(ts, args, ts.Value(keyDB).(*sql.DB))
 }
 
-func (t *liteTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
-	dbURL := fmt.Sprintf("sqlite://file:%s/atlas.sqlite?cache=shared&_fk=1", ts.Getenv("WORK"))
-	cmdCLI(ts, neg, args, dbURL)
+func (t *myTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
+	cmdCLI(ts, neg, args, t.dsn(ts.Getenv("db")), ts.Getenv(atlasPathKey))
 }
 
-func cmdCLI(ts *testscript.TestScript, neg bool, args []string, dbURL string) {
+func (t *pgTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
+	cmdCLI(ts, neg, args, t.dsn(ts.Getenv("db")), ts.Getenv(atlasPathKey))
+}
+
+func (t *liteTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
+	dbURL := fmt.Sprintf("sqlite://file:%s/atlas.sqlite?cache=shared&_fk=1", ts.Getenv("WORK"))
+	cmdCLI(ts, neg, args, dbURL, ts.Getenv(atlasPathKey))
+}
+
+func cmdCLI(ts *testscript.TestScript, neg bool, args []string, dbURL, cliPath string) {
 	var (
-		cliPath = ts.Getenv(cliPathKey)
 		workDir = ts.Getenv("WORK")
+		r       = strings.NewReplacer("URL", dbURL, "$db", ts.Getenv("db"))
 	)
 	for i, arg := range args {
-		args[i] = strings.ReplaceAll(arg, "URL", dbURL)
+		args[i] = r.Replace(arg)
 	}
 	switch l := len(args); {
 	// If command was run with a unix redirect-like suffix.
@@ -435,6 +417,59 @@ func cmdCLI(ts *testscript.TestScript, neg bool, args []string, dbURL string) {
 	}
 }
 
+func (t *myTest) cmdCmpMig(ts *testscript.TestScript, neg bool, args []string) {
+	cmdCmpMig(ts, neg, args)
+}
+
+func (t *pgTest) cmdCmpMig(ts *testscript.TestScript, neg bool, args []string) {
+	cmdCmpMig(ts, neg, args)
+}
+
+func (t *liteTest) cmdCmpMig(ts *testscript.TestScript, neg bool, args []string) {
+	cmdCmpMig(ts, neg, args)
+}
+
+// cmdCmpMig compares a migration file under migrations with a provided file.
+// If the first argument is a filename that does exist, that file is used for comparison.
+// If there is no file with that name, the argument is parsed to an integer n and the
+// nth sql file is used for comparison. Lexicographic order of
+// the files in the directory is used to access the file of interest.
+func cmdCmpMig(ts *testscript.TestScript, _ bool, args []string) {
+	if len(args) < 2 {
+		ts.Fatalf("invalid number of args to 'cmpmig': %d", len(args))
+	}
+	// Check if there is a file prefixed by database version (1.sql and <version>/1.sql).
+	var (
+		ver   = ts.Getenv("version")
+		fname = args[1]
+	)
+	if _, err := os.Stat(ts.MkAbs(filepath.Join(ver, fname))); err == nil {
+		fname = filepath.Join(ver, fname)
+	}
+	expected := ts.ReadFile(fname)
+	dir, err := os.ReadDir(ts.MkAbs("migrations"))
+	ts.Check(err)
+	idx, err := strconv.Atoi(args[0])
+	ts.Check(err)
+	current := 0
+	for _, f := range dir {
+		if f.IsDir() || !strings.HasSuffix(f.Name(), ".sql") {
+			continue
+		}
+		if current == idx {
+			actual := ts.ReadFile(filepath.Join("migrations", f.Name()))
+			var sb strings.Builder
+			if strings.TrimSpace(actual) == strings.TrimSpace(expected) {
+				return
+			}
+			ts.Check(diff.Text(f.Name(), args[1], expected, actual, &sb))
+			ts.Fatalf(sb.String())
+		}
+		current++
+	}
+	ts.Fatalf("could not find the #%d migration", idx)
+}
+
 func cmdExec(ts *testscript.TestScript, args []string, db *sql.DB) {
 	if len(args) == 0 {
 		ts.Fatalf("missing statements for 'execsql'")
@@ -506,7 +541,7 @@ func (t *myTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Chang
 		ctx     = context.Background()
 		r       = strings.NewReplacer("$charset", ts.Getenv("charset"), "$collate", ts.Getenv("collate"), "$db", ts.Getenv("db"))
 	)
-	ts.Check(mysql.UnmarshalHCL([]byte(r.Replace(f)), desired))
+	ts.Check(mysql.EvalHCLBytes([]byte(r.Replace(f)), desired, nil))
 	current, err := t.drv.InspectSchema(ctx, desired.Name, nil)
 	ts.Check(err)
 	desired, err = t.drv.(schema.Normalizer).NormalizeSchema(ctx, desired)
@@ -536,7 +571,7 @@ func (t *pgTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Chang
 		ctx     = context.Background()
 		f       = strings.ReplaceAll(ts.ReadFile(name), "$db", ts.Getenv("db"))
 	)
-	ts.Check(postgres.UnmarshalHCL([]byte(f), desired))
+	ts.Check(postgres.EvalHCLBytes([]byte(f), desired, nil))
 	current, err := t.drv.InspectSchema(ctx, desired.Name, nil)
 	ts.Check(err)
 	desired, err = t.drv.(schema.Normalizer).NormalizeSchema(ctx, desired)
@@ -566,17 +601,37 @@ func (t *liteTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Cha
 		f       = ts.ReadFile(name)
 		drv     = ts.Value(keyDrv).(*sqlite.Driver)
 	)
-	ts.Check(sqlite.UnmarshalHCL([]byte(f), desired))
+	ts.Check(sqlite.EvalHCLBytes([]byte(f), desired, nil))
 	current, err := drv.InspectSchema(context.Background(), desired.Name, nil)
 	ts.Check(err)
 	changes, err := drv.SchemaDiff(current, desired)
-	// Diff errors should returned back to the caller.
+	// Diff errors should return to the caller.
 	if err != nil {
 		return nil, err
 	}
 	return changes, nil
 }
 
+func (t *myTest) clearSchema(ts *testscript.TestScript, _ bool, args []string) {
+	if len(args) == 0 {
+		args = append(args, ts.Getenv("db"))
+	}
+	_, err := t.db.Exec("DROP DATABASE IF EXISTS " + args[0])
+	ts.Check(err)
+	_, err = t.db.Exec("CREATE DATABASE IF NOT EXISTS " + args[0])
+	ts.Check(err)
+}
+
+func (t *pgTest) clearSchema(ts *testscript.TestScript, _ bool, args []string) {
+	if len(args) == 0 {
+		args = append(args, ts.Getenv("db"))
+	}
+	_, err := t.db.Exec(fmt.Sprintf("DROP SCHEMA IF EXISTS %s CASCADE", args[0]))
+	ts.Check(err)
+	_, err = t.db.Exec("CREATE SCHEMA IF NOT EXISTS " + args[0])
+	ts.Check(err)
+}
+
 func cmdSynced(ts *testscript.TestScript, neg bool, args []string, diff func(*testscript.TestScript, string) ([]schema.Change, error)) {
 	if len(args) != 1 {
 		ts.Fatalf("unexpected number of args to synced command: %d", len(args))
@@ -591,7 +646,7 @@ func cmdSynced(ts *testscript.TestScript, neg bool, args []string, diff func(*te
 	}
 }
 
-func cmdApply(ts *testscript.TestScript, neg bool, args []string, apply func(context.Context, []schema.Change) error, diff func(*testscript.TestScript, string) ([]schema.Change, error)) {
+func cmdApply(ts *testscript.TestScript, neg bool, args []string, apply func(context.Context, []schema.Change, ...migrate.PlanOption) error, diff func(*testscript.TestScript, string) ([]schema.Change, error)) {
 	changes, err := diff(ts, args[0])
 	switch {
 	case err != nil && !neg:
