@@ -15,12 +15,9 @@ import (
 	"strings"
 	"testing"
 
-	entmigrate "ariga.io/atlas/cmd/atlascmd/migrate"
-	"ariga.io/atlas/cmd/atlascmd/migrate/ent/revision"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
-	"ariga.io/atlas/sql/sqlclient"
 	"ariga.io/atlas/sql/sqlite"
 
 	"entgo.io/ent/dialect"
@@ -50,23 +47,6 @@ func liteRun(t *testing.T, fn func(test *liteTest)) {
 	fn(tt)
 }
 
-func TestSQLite_EntRevisions(t *testing.T) {
-	liteRun(t, func(t *liteTest) {
-		c, err := sqlclient.Open(context.Background(), t.url())
-		require.NoError(t, err)
-
-		r, err := entmigrate.NewEntRevisions(c)
-		require.NoError(t, err)
-
-		require.NoError(t, r.Init(context.Background()))
-
-		s, err := c.InspectSchema(context.Background(), "", nil)
-		require.NoError(t, err)
-		_, ok := s.Table(revision.Table)
-		require.True(t, ok)
-	})
-}
-
 func TestSQLite_Executor(t *testing.T) {
 	liteRun(t, func(t *liteTest) {
 		testExecutor(t)
@@ -207,6 +187,30 @@ func TestSQLite_AutoIncrement(t *testing.T) {
 	})
 }
 
+func TestSQLite_AutoIncrementSequence(t *testing.T) {
+	// This test shows a bug detected in Ent when working with pre-defined auto-increment start values.
+	// If there is a change somewhere to create an auto-increment with a start value, Atlas must make sure to create
+	// an entry in the 'sqlite_sequence' table (and also ensure the table exists before attempting to create the entry).
+	liteRun(t, func(t *liteTest) {
+		t1 := &schema.Table{
+			Name: "users",
+			Columns: []*schema.Column{
+				{
+					Name:  "id",
+					Type:  &schema.ColumnType{Type: &schema.IntegerType{T: "integer"}},
+					Attrs: []schema.Attr{&sqlite.AutoIncrement{Seq: 10}},
+				},
+			},
+			Attrs: []schema.Attr{&sqlite.AutoIncrement{}},
+		}
+		t1.PrimaryKey = &schema.Index{Table: t1, Parts: []*schema.IndexPart{{C: t1.Columns[0]}}}
+		t1.Columns[0].Indexes = append(t1.Columns[0].Indexes, t1.PrimaryKey)
+
+		// Planning the changes should not result in an error.
+		_ = plan(t, "col_seq", &schema.AddTable{T: t1})
+	})
+}
+
 func TestSQLite_AddColumns(t *testing.T) {
 	liteRun(t, func(t *liteTest) {
 		usersT := &schema.Table{
@@ -466,7 +470,7 @@ create table atlas_defaults
 		spec, err := sqlite.MarshalHCL(realm.Schemas[0])
 		require.NoError(t, err)
 		var s schema.Schema
-		err = sqlite.UnmarshalHCL(spec, &s)
+		err = sqlite.EvalHCLBytes(spec, &s, nil)
 		require.NoError(t, err)
 		t.dropTables(n)
 		t.applyHcl(string(spec))
@@ -500,12 +504,12 @@ env "hello" {
 			})
 			require.NoError(t, err)
 
-			testCLISchemaInspectEnv(t, h, "hello", sqlite.UnmarshalHCL)
+			testCLISchemaInspectEnv(t, h, "hello", sqlite.EvalHCL)
 		})
 	})
 	t.Run("SchemaInspect", func(t *testing.T) {
 		liteRun(t, func(t *liteTest) {
-			testCLISchemaInspect(t, h, t.dsn(), sqlite.UnmarshalHCL)
+			testCLISchemaInspect(t, h, t.dsn(), sqlite.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
@@ -745,7 +749,7 @@ create table atlas_types_sanity
 				},
 				{
 					Name: "tBlob",
-					Type: &schema.ColumnType{Type: &schema.BinaryType{T: "blob", Size: 0}, Raw: "blob(10)", Null: false},
+					Type: &schema.ColumnType{Type: &schema.BinaryType{T: "blob"}, Raw: "blob(10)", Null: false},
 					Default: &schema.Literal{
 						V: "'A'",
 					},
@@ -810,7 +814,7 @@ func (t *liteTest) dropSchemas(...string) {}
 func (t *liteTest) applyHcl(spec string) {
 	realm := t.loadRealm()
 	var desired schema.Schema
-	err := sqlite.UnmarshalHCL([]byte(spec), &desired)
+	err := sqlite.EvalHCLBytes([]byte(spec), &desired, nil)
 	require.NoError(t, err)
 	existing := realm.Schemas[0]
 	diff, err := t.drv.SchemaDiff(existing, &desired)
