@@ -8,9 +8,11 @@ import (
 	"fmt"
 	"testing"
 
+	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/internal/spectest"
+	"ariga.io/atlas/sql/internal/specutil"
 	"ariga.io/atlas/sql/schema"
-
+	"ariga.io/atlas/sql/sqlspec"
 	"github.com/stretchr/testify/require"
 )
 
@@ -102,7 +104,7 @@ table "accounts" {
 }
 `
 	var s schema.Schema
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 
 	exp := &schema.Schema{
@@ -379,7 +381,7 @@ schema "test" {
 			&schema.Collation{V: "utf8mb4_0900_ai_ci"},
 		}
 	)
-	require.NoError(t, UnmarshalHCL(buf, &s2))
+	require.NoError(t, EvalHCLBytes(buf, &s2, nil))
 	require.Equal(t, utf8mb4, s2.Attrs)
 	posts, ok := s2.Table("posts")
 	require.True(t, ok)
@@ -565,7 +567,7 @@ table "users" {
 }
 `
 	)
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 	c := schema.NewStringColumn("name", "text")
 	exp := schema.New("test").
@@ -586,16 +588,23 @@ table "users" {
 
 func TestMarshalSpec_IndexParts(t *testing.T) {
 	c := schema.NewStringColumn("name", "text")
+	c2 := schema.NewStringColumn("Full Name", "text")
 	s := schema.New("test").
 		AddTables(
 			schema.NewTable("users").
-				AddColumns(c).
+				AddColumns(c, c2).
 				AddIndexes(
 					schema.NewIndex("idx").
 						AddParts(
 							schema.NewColumnPart(c).SetDesc(true).AddAttrs(&SubPart{Len: 10}),
 							schema.NewExprPart(&schema.RawExpr{X: "lower(name)"}),
 						),
+					schema.NewIndex("idx2").
+						AddParts(
+							schema.NewColumnPart(c2).SetDesc(true).AddAttrs(&SubPart{Len: 10}),
+						),
+					schema.NewIndex("idx3").
+						AddParts(schema.NewColumnPart(c2)),
 				),
 		)
 	buf, err := MarshalHCL(s)
@@ -606,6 +615,10 @@ func TestMarshalSpec_IndexParts(t *testing.T) {
     null = false
     type = text
   }
+  column "Full Name" {
+    null = false
+    type = text
+  }
   index "idx" {
     on {
       desc   = true
@@ -616,11 +629,21 @@ func TestMarshalSpec_IndexParts(t *testing.T) {
       expr = "lower(name)"
     }
   }
+  index "idx2" {
+    on {
+      desc   = true
+      column = column["Full Name"]
+      prefix = 10
+    }
+  }
+  index "idx3" {
+    columns = [column["Full Name"]]
+  }
 }
 schema "test" {
 }
 `
-	require.EqualValues(t, exp, buf)
+	require.EqualValues(t, exp, string(buf))
 }
 
 func TestMarshalSpec_TimePrecision(t *testing.T) {
@@ -634,7 +657,7 @@ func TestMarshalSpec_TimePrecision(t *testing.T) {
 					schema.NewTimeColumn("tTimestamp", TypeTimestamp, schema.TimePrecision(3)).
 						SetDefault(&schema.RawExpr{X: "current_timestamp(3)"}).
 						AddAttrs(&OnUpdate{A: "current_timestamp(3)"}),
-					schema.NewTimeColumn("tDate", TypeDate, schema.TimePrecision(2)),
+					schema.NewTimeColumn("tDate", TypeDate),
 					schema.NewTimeColumn("tYear", TypeYear, schema.TimePrecision(2)),
 				),
 		)
@@ -662,7 +685,7 @@ func TestMarshalSpec_TimePrecision(t *testing.T) {
   }
   column "tDate" {
     null = false
-    type = date(2)
+    type = date
   }
   column "tYear" {
     null = false
@@ -758,7 +781,7 @@ table "users" {
 }
 `
 	)
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 	exp := schema.New("test").
 		AddTables(
@@ -815,6 +838,7 @@ schema "test" {
 }
 
 func TestTypes(t *testing.T) {
+	p := func(i int) *int { return &i }
 	tests := []struct {
 		typeExpr  string
 		extraAttr string
@@ -834,11 +858,11 @@ func TestTypes(t *testing.T) {
 		},
 		{
 			typeExpr: "binary(255)",
-			expected: &schema.BinaryType{T: TypeBinary, Size: 255},
+			expected: &schema.BinaryType{T: TypeBinary, Size: p(255)},
 		},
 		{
 			typeExpr: "varbinary(255)",
-			expected: &schema.BinaryType{T: TypeVarBinary, Size: 255},
+			expected: &schema.BinaryType{T: TypeVarBinary, Size: p(255)},
 		},
 		{
 			typeExpr: "int",
@@ -910,9 +934,13 @@ func TestTypes(t *testing.T) {
 			expected: &schema.EnumType{T: TypeEnum, Values: []string{"on", "off"}},
 		},
 		{
-			typeExpr: "bit(10)",
+			typeExpr: "bit",
 			expected: &BitType{T: TypeBit},
 		},
+		{
+			typeExpr: "bit(10)",
+			expected: &BitType{T: TypeBit, Size: 10},
+		},
 		{
 			typeExpr: "int(10)",
 			expected: &schema.IntegerType{T: TypeInt},
@@ -1001,10 +1029,6 @@ func TestTypes(t *testing.T) {
 			typeExpr: "date",
 			expected: &schema.TimeType{T: TypeDate},
 		},
-		{
-			typeExpr: "date(2)",
-			expected: typeTime(TypeDate, 2),
-		},
 		{
 			typeExpr: "time",
 			expected: &schema.TimeType{T: TypeTime},
@@ -1039,27 +1063,31 @@ func TestTypes(t *testing.T) {
 		},
 		{
 			typeExpr: "varbinary(30)",
-			expected: &schema.BinaryType{T: TypeVarBinary, Size: 30},
+			expected: &schema.BinaryType{T: TypeVarBinary, Size: p(30)},
+		},
+		{
+			typeExpr: "binary",
+			expected: &schema.BinaryType{T: TypeBinary},
 		},
 		{
 			typeExpr: "binary(5)",
-			expected: &schema.BinaryType{T: TypeBinary, Size: 5},
+			expected: &schema.BinaryType{T: TypeBinary, Size: p(5)},
 		},
 		{
 			typeExpr: "blob(5)",
-			expected: &schema.StringType{T: TypeBlob},
+			expected: &schema.BinaryType{T: TypeBlob},
 		},
 		{
 			typeExpr: "tinyblob",
-			expected: &schema.StringType{T: TypeTinyBlob},
+			expected: &schema.BinaryType{T: TypeTinyBlob},
 		},
 		{
 			typeExpr: "mediumblob",
-			expected: &schema.StringType{T: TypeMediumBlob},
+			expected: &schema.BinaryType{T: TypeMediumBlob},
 		},
 		{
 			typeExpr: "longblob",
-			expected: &schema.StringType{T: TypeLongBlob},
+			expected: &schema.BinaryType{T: TypeLongBlob},
 		},
 		{
 			typeExpr: "json",
@@ -1143,14 +1171,14 @@ schema "test" {
 }
 `, tt.typeExpr, lineIfSet(tt.extraAttr))
 			var test schema.Schema
-			err := UnmarshalHCL([]byte(doc), &test)
+			err := EvalHCLBytes([]byte(doc), &test, nil)
 			require.NoError(t, err)
 			colspec := test.Tables[0].Columns[0]
 			require.EqualValues(t, tt.expected, colspec.Type.Type)
 			spec, err := MarshalHCL(&test)
 			require.NoError(t, err)
 			var after schema.Schema
-			err = UnmarshalHCL(spec, &after)
+			err = EvalHCLBytes(spec, &after, nil)
 			require.NoError(t, err)
 			require.EqualValues(t, tt.expected, after.Tables[0].Columns[0].Type.Type)
 		})
@@ -1161,6 +1189,78 @@ func TestInputVars(t *testing.T) {
 	spectest.TestInputVars(t, EvalHCL)
 }
 
+func TestQualifyReferencedTables(t *testing.T) {
+	var (
+		col       = schema.NewIntColumn("col", "integer")
+		targetCol = schema.NewIntColumn("target_column", "integer")
+		targetTbl = schema.NewTable("target_table").AddColumns(targetCol)
+		realm     = schema.NewRealm(
+			schema.New("target_schema").AddTables(targetTbl),
+			schema.New("sch").
+				AddTables(
+					schema.NewTable("tbl").
+						AddColumns(col).
+						AddForeignKeys(
+							schema.NewForeignKey("col_fk").
+								SetRefTable(targetTbl).
+								AddColumns(col).
+								AddRefColumns(targetCol),
+						),
+				),
+		)
+		tables = []*sqlspec.Table{
+			{
+				Name:   "tbl",
+				Schema: &schemahcl.Ref{V: "$schema.sch"},
+			},
+			{
+				Name:   "target_table",
+				Schema: &schemahcl.Ref{V: "$schema.target_schema"},
+			},
+		}
+	)
+	require.NoError(t, specutil.QualifyReferencedTables(tables, realm))
+	require.Zero(t, tables[0].Qualifier)
+	require.Equal(t, "target_schema", tables[1].Qualifier)
+}
+
+func TestParseType_Decimal(t *testing.T) {
+	for _, tt := range []struct {
+		input   string
+		wantT   *schema.DecimalType
+		wantErr bool
+	}{
+		{
+			input: "decimal",
+			wantT: &schema.DecimalType{T: TypeDecimal},
+		},
+		{
+			input: "decimal unsigned",
+			wantT: &schema.DecimalType{T: TypeDecimal, Unsigned: true},
+		},
+		{
+			input: "decimal(10)",
+			wantT: &schema.DecimalType{T: TypeDecimal, Precision: 10},
+		},
+		{
+			input: "decimal(10) unsigned",
+			wantT: &schema.DecimalType{T: TypeDecimal, Precision: 10, Unsigned: true},
+		},
+		{
+			input: "decimal(10,2)",
+			wantT: &schema.DecimalType{T: TypeDecimal, Precision: 10, Scale: 2},
+		},
+		{
+			input: "decimal(10, 2) unsigned",
+			wantT: &schema.DecimalType{T: TypeDecimal, Precision: 10, Scale: 2, Unsigned: true},
+		},
+	} {
+		d, err := ParseType(tt.input)
+		require.Equal(t, tt.wantErr, err != nil)
+		require.Equal(t, tt.wantT, d)
+	}
+}
+
 func typeTime(t string, p int) schema.Type {
 	return &schema.TimeType{T: t, Precision: &p}
 }
