@@ -41,7 +41,7 @@ func (d *diff) TableAttrDiff(from, to *schema.Table) ([]schema.Change, error) {
 }
 
 // ColumnChange returns the schema changes (if any) for migrating one column to the other.
-func (d *diff) ColumnChange(from, to *schema.Column) (schema.ChangeKind, error) {
+func (d *diff) ColumnChange(_ *schema.Table, from, to *schema.Column) (schema.ChangeKind, error) {
 	change := sqlx.CommentChange(from.Attrs, to.Attrs)
 	if from.Type.Null != to.Type.Null {
 		change |= schema.ChangeNull
@@ -196,32 +196,14 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 	if fromT == nil || toT == nil {
 		return false, fmt.Errorf("postgres: missing type information for column %q", from.Name)
 	}
-	// Skip checking SERIAL types as they are not real types in the database, but more
-	// like a convenience way for creating integers types with AUTO_INCREMENT property.
-	if s, ok := to.Type.Type.(*SerialType); ok {
-		i, ok := from.Type.Type.(*schema.IntegerType)
-		if !ok {
-			return true, nil
-		}
-		var it string
-		switch s.T {
-		case TypeSmallSerial:
-			it = TypeSmallInt
-		case TypeSerial:
-			it = TypeInteger
-		case TypeBigSerial:
-			it = TypeBigInt
-		}
-		return i.T != it, nil
-	}
 	if reflect.TypeOf(fromT) != reflect.TypeOf(toT) {
 		return true, nil
 	}
 	var changed bool
 	switch fromT := fromT.(type) {
-	case *schema.BinaryType, *schema.BoolType, *schema.DecimalType, *schema.FloatType,
-		*schema.IntegerType, *schema.JSONType, *schema.SpatialType, *schema.StringType,
-		*schema.TimeType, *BitType, *NetworkType, *UserDefinedType:
+	case *schema.BinaryType, *BitType, *schema.BoolType, *schema.DecimalType, *schema.FloatType,
+		*IntervalType, *schema.IntegerType, *schema.JSONType, *SerialType, *schema.SpatialType,
+		*schema.StringType, *schema.TimeType, *NetworkType, *UserDefinedType:
 		t1, err := FormatType(toT)
 		if err != nil {
 			return false, err
@@ -231,12 +213,11 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 			return false, err
 		}
 		changed = t1 != t2
-	case *enumType:
-		toT := toT.(*schema.EnumType)
-		changed = fromT.T != toT.T || !sqlx.ValuesEqual(fromT.Values, toT.Values)
 	case *schema.EnumType:
 		toT := toT.(*schema.EnumType)
-		changed = fromT.T != toT.T || !sqlx.ValuesEqual(fromT.Values, toT.Values)
+		// Column type was changed if the underlying enum type was changed or values are not equal.
+		changed = !sqlx.ValuesEqual(fromT.Values, toT.Values) || fromT.T != toT.T ||
+			(toT.Schema != nil && fromT.Schema != nil && fromT.Schema.Name != toT.Schema.Name)
 	case *CurrencyType:
 		toT := toT.(*CurrencyType)
 		changed = fromT.T != toT.T
@@ -248,12 +229,24 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 		changed = fromT.T != toT.T
 	case *ArrayType:
 		toT := toT.(*ArrayType)
-		changed = fromT.T != toT.T
-		// Array types can be defined differently, but they may represent the same type.
-		// Therefore, in case of mismatch, we verify it using the database engine.
-		if changed {
-			equals, err := d.typesEqual(fromT.T, toT.T)
-			return !equals, err
+		// Same type.
+		if changed = fromT.T != toT.T; !changed {
+			break
+		}
+		// In case the desired schema is not normalized, the string type can look different even
+		// if the two strings represent the same array type (varchar(1), character varying (1)).
+		// Therefore, we try by comparing the underlying types if they were defined.
+		if fromT.Type != nil && toT.Type != nil {
+			t1, err := FormatType(fromT.Type)
+			if err != nil {
+				return false, err
+			}
+			t2, err := FormatType(toT.Type)
+			if err != nil {
+				return false, err
+			}
+			// Same underlying type.
+			changed = t1 != t2
 		}
 	default:
 		return false, &sqlx.UnsupportedTypeError{Type: fromT}
@@ -261,61 +254,6 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 	return changed, nil
 }
 
-// Normalize implements the sqlx.Normalizer interface.
-func (d *diff) Normalize(from, to *schema.Table) {
-	d.normalize(from)
-	d.normalize(to)
-}
-
-func (d *diff) normalize(table *schema.Table) {
-	for _, c := range table.Columns {
-		switch t := c.Type.Type.(type) {
-		case nil:
-		case *schema.TimeType:
-			// "timestamp" and "timestamptz" are accepted as
-			// abbreviations for timestamp with(out) time zone.
-			switch t.T {
-			case "timestamp with time zone":
-				t.T = "timestamptz"
-			case "timestamp without time zone":
-				t.T = "timestamp"
-			}
-		case *schema.FloatType:
-			// The same numeric precision is used in all platform.
-			// See: https://www.postgresql.org/docs/current/datatype-numeric.html
-			switch {
-			case t.T == "float" && t.Precision < 25:
-				// float(1) to float(24) are selected as "real" type.
-				t.T = "real"
-				fallthrough
-			case t.T == "real":
-				t.Precision = 24
-			case t.T == "float" && t.Precision >= 25:
-				// float(25) to float(53) are selected as "double precision" type.
-				t.T = "double precision"
-				fallthrough
-			case t.T == "double precision":
-				t.Precision = 53
-			}
-		case *schema.StringType:
-			switch t.T {
-			case "character", "char":
-				// Character without length specifier
-				// is equivalent to character(1).
-				t.Size = 1
-			}
-		case *enumType:
-			c.Type.Type = &schema.EnumType{T: t.T, Values: t.Values}
-		case *SerialType:
-			// The definition of "<column> <serial type>" is equivalent to specifying:
-			// "<column> <int type> NOT NULL DEFAULT nextval('<table>_<column>_seq')".
-			c.Default = &schema.RawExpr{
-				X: fmt.Sprintf("nextval('%s_%s_seq'::regclass)", table.Name, c.Name),
-			}
-		}
-	}
-}
-
 // valuesEqual reports if the DEFAULT values x and y
 // equal according to the database engine.
 func (d *diff) valuesEqual(x, y string) (bool, error) {
@@ -332,22 +270,6 @@ func (d *diff) valuesEqual(x, y string) (bool, error) {
 	return b, nil
 }
 
-// typesEqual reports if the data types x and y
-// equal according to the database engine.
-func (d *diff) typesEqual(x, y string) (bool, error) {
-	var b bool
-	// The datatype are safe to be inlined in the SELECT statement
-	// same as we inline them in the CREATE TABLE statement.
-	rows, err := d.QueryContext(context.Background(), fmt.Sprintf("SELECT '%s'::regtype = '%s'::regtype", x, y))
-	if err != nil {
-		return false, err
-	}
-	if err := sqlx.ScanOne(rows, &b); err != nil {
-		return false, err
-	}
-	return b, nil
-}
-
 // Default IDENTITY attributes.
 const (
 	defaultIdentityGen  = "BY DEFAULT"
@@ -389,7 +311,8 @@ func identity(attrs []schema.Attr) (*Identity, bool) {
 // formatPartition returns the string representation of the
 // partition key according to the PostgreSQL format/grammar.
 func formatPartition(p Partition) (string, error) {
-	b := Build("PARTITION BY")
+	b := &sqlx.Builder{QuoteChar: '"'}
+	b.P("PARTITION BY")
 	switch t := strings.ToUpper(p.T); t {
 	case PartitionTypeRange, PartitionTypeList, PartitionTypeHash:
 		b.P(t)
