@@ -6,6 +6,7 @@ package postgres
 
 import (
 	"context"
+	"database/sql"
 	"errors"
 	"fmt"
 	"hash/fnv"
@@ -27,6 +28,7 @@ type (
 		schema.Differ
 		schema.Inspector
 		migrate.PlanApplier
+		schema string // the schema given in the `search_path` parameter (if given)
 	}
 
 	// database connection and its information.
@@ -40,15 +42,42 @@ type (
 	}
 )
 
+// DriverName holds the name used for registration.
+const DriverName = "postgres"
+
 func init() {
 	sqlclient.Register(
-		"postgres",
-		sqlclient.DriverOpener(Open),
+		DriverName,
+		sqlclient.OpenerFunc(opener),
+		sqlclient.RegisterDriverOpener(Open),
+		sqlclient.RegisterFlavours("postgresql"),
 		sqlclient.RegisterCodec(MarshalHCL, EvalHCL),
 		sqlclient.RegisterURLParser(parser{}),
 	)
 }
 
+func opener(_ context.Context, u *url.URL) (*sqlclient.Client, error) {
+	ur := parser{}.ParseURL(u)
+	db, err := sql.Open(DriverName, ur.DSN)
+	if err != nil {
+		return nil, err
+	}
+	drv, err := Open(db)
+	if err != nil {
+		if cerr := db.Close(); cerr != nil {
+			err = fmt.Errorf("%w: %v", err, cerr)
+		}
+		return nil, err
+	}
+	drv.(*Driver).schema = ur.Schema
+	return &sqlclient.Client{
+		Name:   DriverName,
+		DB:     db,
+		URL:    ur,
+		Driver: drv,
+	}, nil
+}
+
 // Open opens a new PostgreSQL driver.
 func Open(db schema.ExecQuerier) (migrate.Driver, error) {
 	c := conn{ExecQuerier: db}
@@ -131,6 +160,89 @@ func (d *Driver) Lock(ctx context.Context, name string, timeout time.Duration) (
 	}, nil
 }
 
+// Snapshot implements migrate.Snapshoter.
+func (d *Driver) Snapshot(ctx context.Context) (migrate.RestoreFunc, error) {
+	// Postgres will only then be considered bound to a schema if the `search_path` was given.
+	// In all other cases, the connection is considered bound to the realm.
+	if d.schema != "" {
+		s, err := d.InspectSchema(ctx, d.schema, nil)
+		if err != nil {
+			return nil, err
+		}
+		if len(s.Tables) > 0 {
+			return nil, migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in connected schema", s.Tables[0].Name)}
+		}
+		return func(ctx context.Context) error {
+			current, err := d.InspectSchema(ctx, s.Name, nil)
+			if err != nil {
+				return err
+			}
+			changes, err := d.SchemaDiff(current, s)
+			if err != nil {
+				return err
+			}
+			return d.ApplyChanges(ctx, changes)
+		}, nil
+	}
+	// Not bound to a schema.
+	realm, err := d.InspectRealm(ctx, nil)
+	if err != nil {
+		return nil, err
+	}
+	restore := func(ctx context.Context) error {
+		current, err := d.InspectRealm(ctx, nil)
+		if err != nil {
+			return err
+		}
+		changes, err := d.RealmDiff(current, realm)
+		if err != nil {
+			return err
+		}
+		return d.ApplyChanges(ctx, changes)
+	}
+	// Postgres is considered clean, if there are no schemas or the public schema has no tables.
+	if len(realm.Schemas) == 0 {
+		return restore, nil
+	}
+	if s, ok := realm.Schema("public"); len(realm.Schemas) == 1 && ok {
+		if len(s.Tables) > 0 {
+			return nil, migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+		}
+		return restore, nil
+	}
+	return nil, migrate.NotCleanError{Reason: fmt.Sprintf("found schema %q", realm.Schemas[0].Name)}
+}
+
+// CheckClean implements migrate.CleanChecker.
+func (d *Driver) CheckClean(ctx context.Context, revT *migrate.TableIdent) error {
+	if d.schema != "" {
+		switch s, err := d.InspectSchema(ctx, d.schema, nil); {
+		case err != nil:
+			return err
+		case len(s.Tables) == 0, (revT.Schema == "" || s.Name == revT.Schema) && len(s.Tables) == 1 && s.Tables[0].Name == revT.Name:
+			return nil
+		default:
+			return &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+		}
+	}
+	r, err := d.InspectRealm(ctx, nil)
+	if err != nil {
+		return err
+	}
+	for _, s := range r.Schemas {
+		switch {
+		case len(s.Tables) == 0 && s.Name == "public":
+		case len(s.Tables) == 0 || s.Name != revT.Schema:
+			return &migrate.NotCleanError{Reason: fmt.Sprintf("found schema %q", s.Name)}
+		case len(s.Tables) > 1:
+			return &migrate.NotCleanError{Reason: fmt.Sprintf("found %d tables in schema %q", len(s.Tables), s.Name)}
+		case len(s.Tables) == 1 && s.Tables[0].Name != revT.Name:
+			return &migrate.NotCleanError{Reason: fmt.Sprintf("found table %q in schema %q", s.Tables[0].Name, s.Name)}
+		}
+	}
+	return nil
+}
+
 func acquire(ctx context.Context, conn schema.ExecQuerier, id uint32, timeout time.Duration) error {
 	switch {
 	// With timeout (context-based).
@@ -232,6 +344,7 @@ const (
 	TypeReal   = "real"
 	TypeFloat8 = "float8" // double precision
 	TypeFloat4 = "float4" // real
+	TypeFloat  = "float"  // float(p).
 
 	TypeNumeric = "numeric"
 	TypeDecimal = "decimal" // numeric
