@@ -8,6 +8,7 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
+	"regexp"
 	"strconv"
 	"strings"
 
@@ -26,16 +27,19 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 	if err != nil {
 		return nil, err
 	}
+	if opts == nil {
+		opts = &schema.InspectRealmOption{}
+	}
 	r := schema.NewRealm(schemas...).SetCollation(i.collate)
 	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
 	if len(schemas) == 0 || !sqlx.ModeInspectRealm(opts).Is(schema.InspectTables) {
-		return r, nil
+		return sqlx.ExcludeRealm(r, opts.Exclude)
 	}
 	if err := i.inspectTables(ctx, r, nil); err != nil {
 		return nil, err
 	}
 	sqlx.LinkSchemaTables(schemas)
-	return r, nil
+	return sqlx.ExcludeRealm(r, opts.Exclude)
 }
 
 // InspectSchema returns schema descriptions of the tables in the given schema.
@@ -51,6 +55,9 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 	case n > 1:
 		return nil, fmt.Errorf("postgres: %d schemas were found for %q", n, name)
 	}
+	if opts == nil {
+		opts = &schema.InspectOptions{}
+	}
 	r := schema.NewRealm(schemas...).SetCollation(i.collate)
 	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
 	if sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
@@ -59,7 +66,7 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 		}
 		sqlx.LinkSchemaTables(schemas)
 	}
-	return r.Schemas[0], nil
+	return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
 }
 
 func (i *inspect) inspectTables(ctx context.Context, r *schema.Realm, opts *schema.InspectOptions) error {
@@ -163,14 +170,14 @@ func (i *inspect) columns(ctx context.Context, s *schema.Schema) error {
 }
 
 // addColumn scans the current row and adds a new column from it to the table.
-func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) error {
+func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) (err error) {
 	var (
-		typid, maxlen, precision, timeprecision, scale, seqstart, seqinc                                              sql.NullInt64
-		table, name, typ, nullable, defaults, udt, identity, genidentity, genexpr, charset, collate, comment, typtype sql.NullString
+		typid, maxlen, precision, timeprecision, scale, seqstart, seqinc, seqlast                                                  sql.NullInt64
+		table, name, typ, fmtype, nullable, defaults, identity, genidentity, genexpr, charset, collate, comment, typtype, interval sql.NullString
 	)
-	if err := rows.Scan(
-		&table, &name, &typ, &nullable, &defaults, &maxlen, &precision, &timeprecision, &scale, &charset,
-		&collate, &udt, &identity, &seqstart, &seqinc, &genidentity, &genexpr, &comment, &typtype, &typid,
+	if err = rows.Scan(
+		&table, &name, &typ, &fmtype, &nullable, &defaults, &maxlen, &precision, &timeprecision, &scale, &interval,
+		&charset, &collate, &identity, &seqstart, &seqinc, &seqlast, &genidentity, &genexpr, &comment, &typtype, &typid,
 	); err != nil {
 		return err
 	}
@@ -185,23 +192,25 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) error {
 			Null: nullable.String == "YES",
 		},
 	}
-	c.Type.Type = columnType(&columnDesc{
+	c.Type.Type, err = columnType(&columnDesc{
 		typ:           typ.String,
+		fmtype:        fmtype.String,
 		size:          maxlen.Int64,
-		udt:           udt.String,
 		scale:         scale.Int64,
 		typtype:       typtype.String,
 		typid:         typid.Int64,
+		interval:      interval.String,
 		precision:     precision.Int64,
 		timePrecision: &timeprecision.Int64,
 	})
 	if defaults.Valid {
-		c.Default = defaultExpr(c, defaults.String)
+		defaultExpr(c, defaults.String)
 	}
 	if identity.String == "YES" {
 		c.Attrs = append(c.Attrs, &Identity{
 			Generation: genidentity.String,
 			Sequence: &Sequence{
+				Last:      seqlast.Int64,
 				Start:     seqstart.Int64,
 				Increment: seqinc.Int64,
 			},
@@ -225,71 +234,6 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) error {
 	return nil
 }
 
-func columnType(c *columnDesc) schema.Type {
-	var typ schema.Type
-	switch t := c.typ; strings.ToLower(t) {
-	case TypeBigInt, TypeInt8, TypeInt, TypeInteger, TypeInt4, TypeSmallInt, TypeInt2, TypeInt64:
-		typ = &schema.IntegerType{T: t}
-	case TypeBit, TypeBitVar:
-		typ = &BitType{T: t, Len: c.size}
-	case TypeBool, TypeBoolean:
-		typ = &schema.BoolType{T: t}
-	case TypeBytea:
-		typ = &schema.BinaryType{T: t}
-	case TypeCharacter, TypeChar, TypeCharVar, TypeVarChar, TypeText:
-		// A `character` column without length specifier is equivalent to `character(1)`,
-		// but `varchar` without length accepts strings of any size (same as `text`).
-		typ = &schema.StringType{T: t, Size: int(c.size)}
-	case TypeCIDR, TypeInet, TypeMACAddr, TypeMACAddr8:
-		typ = &NetworkType{T: t}
-	case TypeCircle, TypeLine, TypeLseg, TypeBox, TypePath, TypePolygon, TypePoint, TypeGeometry:
-		typ = &schema.SpatialType{T: t}
-	case TypeDate:
-		typ = &schema.TimeType{T: t}
-	case TypeTime, TypeTimeWOTZ, TypeTimeTZ, TypeTimeWTZ, TypeTimestamp,
-		TypeTimestampTZ, TypeTimestampWTZ, TypeTimestampWOTZ:
-		p := defaultTimePrecision
-		if c.timePrecision != nil {
-			p = int(*c.timePrecision)
-		}
-		typ = &schema.TimeType{T: t, Precision: &p}
-	case TypeInterval:
-		// TODO: get 'interval_type' from query above before implementing.
-		typ = &schema.UnsupportedType{T: t}
-	case TypeReal, TypeDouble, TypeFloat4, TypeFloat8:
-		typ = &schema.FloatType{T: t, Precision: int(c.precision)}
-	case TypeJSON, TypeJSONB:
-		typ = &schema.JSONType{T: t}
-	case TypeMoney:
-		typ = &CurrencyType{T: t}
-	case TypeDecimal, TypeNumeric:
-		typ = &schema.DecimalType{T: t, Precision: int(c.precision), Scale: int(c.scale)}
-	case TypeSmallSerial, TypeSerial, TypeBigSerial, TypeSerial2, TypeSerial4, TypeSerial8:
-		typ = &SerialType{T: t, Precision: int(c.precision)}
-	case TypeUUID:
-		typ = &UUIDType{T: t}
-	case TypeXML:
-		typ = &XMLType{T: t}
-	case TypeArray:
-		// Note that for ARRAY types, the 'udt_name' column holds the array type
-		// prefixed with '_'. For example, for 'integer[]' the result is '_int',
-		// and for 'text[N][M]' the result is also '_text'. That's because, the
-		// database ignores any size or multi-dimensions constraints.
-		typ = &ArrayType{T: strings.TrimPrefix(c.udt, "_") + "[]"}
-	case TypeUserDefined:
-		typ = &UserDefinedType{T: c.udt}
-		// The `typtype` column is set to 'e' for enum types, and the
-		// values are filled in batch after the rows above is closed.
-		// https://www.postgresql.org/docs/current/catalog-pg-type.html
-		if c.typtype == "e" {
-			typ = &enumType{T: c.udt, ID: c.typid}
-		}
-	default:
-		typ = &schema.UnsupportedType{T: t}
-	}
-	return typ
-}
-
 // enumValues fills enum columns with their values from the database.
 func (i *inspect) enumValues(ctx context.Context, s *schema.Schema) error {
 	var (
@@ -303,9 +247,12 @@ func (i *inspect) enumValues(ctx context.Context, s *schema.Schema) error {
 				if _, ok := ids[enum.ID]; !ok {
 					args = append(args, enum.ID)
 				}
-				// Convert the intermediate type to the
-				// standard schema.EnumType.
-				e := &schema.EnumType{T: enum.T}
+				// Convert the intermediate type to
+				// the standard schema.EnumType.
+				e := &schema.EnumType{T: enum.T, Schema: s}
+				if enum.Schema != "" && enum.Schema != s.Name {
+					e.Schema = schema.New(enum.Schema)
+				}
 				c.Type.Type = e
 				c.Type.Raw = enum.T
 				ids[enum.ID] = append(ids[enum.ID], e)
@@ -597,31 +544,44 @@ func nArgs(start, n int) string {
 	return b.String()
 }
 
-func defaultExpr(c *schema.Column, x string) schema.Expr {
-	switch {
-	case sqlx.IsLiteralBool(x), sqlx.IsLiteralNumber(x), sqlx.IsQuoted(x, '\''):
-		return &schema.Literal{V: x}
-	default:
-		// Try casting or fallback to raw expressions (e.g. column text[] has the default of '{}':text[]).
-		if v, ok := canConvert(c.Type, x); ok {
-			return &schema.Literal{V: v}
+var reNextval = regexp.MustCompile(`(?i) *nextval\('(?:[\w$]+\.)*([\w$]+_[\w$]+_seq)'(?:::regclass)*\) *$`)
+
+func defaultExpr(c *schema.Column, s string) {
+	switch m := reNextval.FindStringSubmatch(s); {
+	// The definition of "<column> <serial type>" is equivalent to specifying:
+	// "<column> <int type> NOT NULL DEFAULT nextval('<table>_<column>_seq')".
+	// https://postgresql.org/docs/current/datatype-numeric.html#DATATYPE-SERIAL.
+	case len(m) == 2:
+		tt, ok := c.Type.Type.(*schema.IntegerType)
+		if !ok {
+			return
 		}
-		return &schema.RawExpr{X: x}
+		st := &SerialType{SequenceName: m[1]}
+		st.SetType(tt)
+		c.Type.Raw = st.T
+		c.Type.Type = st
+	case sqlx.IsLiteralBool(s), sqlx.IsLiteralNumber(s), sqlx.IsQuoted(s, '\''):
+		c.Default = &schema.Literal{V: s}
+	default:
+		var x schema.Expr = &schema.RawExpr{X: s}
+		// Try casting or fallback to raw expressions (e.g. column text[] has the default of '{}':text[]).
+		if v, ok := canConvert(c.Type, s); ok {
+			x = &schema.Literal{V: v}
+		}
+		c.Default = x
 	}
 }
 
 func canConvert(t *schema.ColumnType, x string) (string, bool) {
-	r := t.Raw
-	if t, ok := t.Type.(*ArrayType); ok {
-		r = t.T
-	}
-	i := strings.Index(x, "::"+r)
+	i := strings.LastIndex(x, "::")
 	if i == -1 || !sqlx.IsQuoted(x[:i], '\'') {
 		return "", false
 	}
 	q := x[0:i]
 	x = x[1 : i-1]
 	switch t.Type.(type) {
+	case *enumType:
+		return q, true
 	case *schema.BoolType:
 		if sqlx.IsLiteralBool(x) {
 			return x, true
@@ -654,27 +614,37 @@ type (
 	enumType struct {
 		schema.Type
 		T      string // Type name.
+		Schema string // Optional schema name.
 		ID     int64  // Type id.
 		Values []string
 	}
 
 	// ArrayType defines an array type.
-	// https://www.postgresql.org/docs/current/arrays.html
+	// https://postgresql.org/docs/current/arrays.html
 	ArrayType struct {
-		schema.Type
-		T string
+		schema.Type        // Underlying items type (e.g. varchar(255)).
+		T           string // Formatted type (e.g. int[]).
 	}
 
 	// BitType defines a bit type.
-	// https://www.postgresql.org/docs/current/datatype-bit.html
+	// https://postgresql.org/docs/current/datatype-bit.html
 	BitType struct {
 		schema.Type
 		T   string
 		Len int64
 	}
 
+	// IntervalType defines an interval type.
+	// https://postgresql.org/docs/current/datatype-datetime.html
+	IntervalType struct {
+		schema.Type
+		T         string // Type name.
+		F         string // Optional field. YEAR, MONTH, ..., MINUTE TO SECOND.
+		Precision *int   // Optional precision.
+	}
+
 	// A NetworkType defines a network type.
-	// https://www.postgresql.org/docs/current/datatype-net-types.html
+	// https://postgresql.org/docs/current/datatype-net-types.html
 	NetworkType struct {
 		schema.Type
 		T   string
@@ -688,10 +658,15 @@ type (
 	}
 
 	// A SerialType defines a serial type.
+	// https://postgresql.org/docs/current/datatype-numeric.html#DATATYPE-SERIAL
 	SerialType struct {
 		schema.Type
 		T         string
 		Precision int
+		// SequenceName holds the inspected sequence name attached to the column.
+		// It defaults to <Table>_<Column>_seq when the column is created, but may
+		// be different in case the table or the column was renamed.
+		SequenceName string
 	}
 
 	// A UUIDType defines a UUID type.
@@ -707,16 +682,19 @@ type (
 	}
 
 	// ConType describes constraint type.
-	// https://www.postgresql.org/docs/current/catalog-pg-constraint.html
+	// https://postgresql.org/docs/current/catalog-pg-constraint.html
 	ConType struct {
 		schema.Attr
 		T string // c, f, p, u, t, x.
 	}
 
 	// Sequence defines (the supported) sequence options.
-	// https://www.postgresql.org/docs/current/sql-createsequence.html
+	// https://postgresql.org/docs/current/sql-createsequence.html
 	Sequence struct {
 		Start, Increment int64
+		// Last sequence value written to disk.
+		// https://postgresql.org/docs/current/view-pg-sequences.html.
+		Last int64
 	}
 
 	// Identity defines an identity column.
@@ -727,21 +705,21 @@ type (
 	}
 
 	// IndexType represents an index type.
-	// https://www.postgresql.org/docs/current/indexes-types.html
+	// https://postgresql.org/docs/current/indexes-types.html
 	IndexType struct {
 		schema.Attr
 		T string // BTREE, BRIN, HASH, GiST, SP-GiST, GIN.
 	}
 
 	// IndexPredicate describes a partial index predicate.
-	// https://www.postgresql.org/docs/current/catalog-pg-index.html
+	// https://postgresql.org/docs/current/catalog-pg-index.html
 	IndexPredicate struct {
 		schema.Attr
 		P string
 	}
 
 	// IndexColumnProperty describes an index column property.
-	// https://www.postgresql.org/docs/current/functions-info.html#FUNCTIONS-INFO-INDEX-COLUMN-PROPS
+	// https://postgresql.org/docs/current/functions-info.html#FUNCTIONS-INFO-INDEX-COLUMN-PROPS
 	IndexColumnProperty struct {
 		schema.Attr
 		// NullsFirst defaults to true for DESC indexes.
@@ -751,7 +729,7 @@ type (
 	}
 
 	// IndexStorageParams describes index storage parameters add with the WITH clause.
-	// https://www.postgresql.org/docs/current/sql-createindex.html#SQL-CREATEINDEX-STORAGE-PARAMETERS
+	// https://postgresql.org/docs/current/sql-createindex.html#SQL-CREATEINDEX-STORAGE-PARAMETERS
 	IndexStorageParams struct {
 		schema.Attr
 		// AutoSummarize defines the authsummarize storage parameter.
@@ -762,7 +740,7 @@ type (
 	}
 
 	// NoInherit attribute defines the NO INHERIT flag for CHECK constraint.
-	// https://www.postgresql.org/docs/current/catalog-pg-constraint.html
+	// https://postgresql.org/docs/current/catalog-pg-constraint.html
 	NoInherit struct {
 		schema.Attr
 	}
@@ -798,6 +776,42 @@ type (
 	}
 )
 
+// IsUnique reports if the type is unique constraint.
+func (c ConType) IsUnique() bool { return strings.ToLower(c.T) == "u" }
+
+// IntegerType returns the underlying integer type this serial type represents.
+func (s *SerialType) IntegerType() *schema.IntegerType {
+	t := &schema.IntegerType{T: TypeInteger}
+	switch s.T {
+	case TypeSerial2, TypeSmallSerial:
+		t.T = TypeSmallInt
+	case TypeSerial8, TypeBigSerial:
+		t.T = TypeBigInt
+	}
+	return t
+}
+
+// SetType sets the serial type from the given integer type.
+func (s *SerialType) SetType(t *schema.IntegerType) {
+	switch t.T {
+	case TypeSmallInt, TypeInt2:
+		s.T = TypeSmallSerial
+	case TypeInteger, TypeInt4, TypeInt:
+		s.T = TypeSerial
+	case TypeBigInt, TypeInt8:
+		s.T = TypeBigSerial
+	}
+}
+
+// sequence returns the inspected name of the sequence
+// or the standard name defined by postgres.
+func (s *SerialType) sequence(t *schema.Table, c *schema.Column) string {
+	if s.SequenceName != "" {
+		return s.SequenceName
+	}
+	return fmt.Sprintf("%s_%s_seq", t.Name, c.Name)
+}
+
 // newIndexStorage parses and returns the index storage parameters.
 func newIndexStorage(opts string) (*IndexStorageParams, error) {
 	params := &IndexStorageParams{}
@@ -824,6 +838,30 @@ func newIndexStorage(opts string) (*IndexStorageParams, error) {
 	return params, nil
 }
 
+// reEnumType extracts the enum type and an option schema qualifier.
+var reEnumType = regexp.MustCompile(`^(?:(".+"|\w+)\.)?(".+"|\w+)$`)
+
+func newEnumType(t string, id int64) *enumType {
+	var (
+		e     = &enumType{T: t, ID: id}
+		parts = reEnumType.FindStringSubmatch(e.T)
+		r     = func(s string) string {
+			s = strings.ReplaceAll(s, `""`, `"`)
+			if len(s) > 1 && s[0] == '"' && s[len(s)-1] == '"' {
+				s = s[1 : len(s)-1]
+			}
+			return s
+		}
+	)
+	if len(parts) > 1 {
+		e.Schema = r(parts[1])
+	}
+	if len(parts) > 2 {
+		e.T = r(parts[2])
+	}
+	return e
+}
+
 const (
 	// Query to list runtime parameters.
 	paramsQuery = `SELECT setting FROM pg_settings WHERE name IN ('lc_collate', 'lc_ctype', 'server_version_num', 'crdb_version') ORDER BY name DESC`
@@ -882,18 +920,20 @@ SELECT
 	t1.table_name,
 	t1.column_name,
 	t1.data_type,
+	pg_catalog.format_type(a.atttypid, a.atttypmod) AS format_type,
 	t1.is_nullable,
 	t1.column_default,
 	t1.character_maximum_length,
 	t1.numeric_precision,
 	t1.datetime_precision,
 	t1.numeric_scale,
+	t1.interval_type,
 	t1.character_set_name,
 	t1.collation_name,
-	t1.udt_name,
 	t1.is_identity,
 	t1.identity_start,
 	t1.identity_increment,
+	(CASE WHEN t1.is_identity = 'YES' THEN (SELECT last_value FROM pg_sequences WHERE quote_ident(schemaname) || '.' || quote_ident(sequencename) = pg_get_serial_sequence(quote_ident(t1.table_schema) || '.' || quote_ident(t1.table_name), t1.column_name)) END) AS identity_last,
 	t1.identity_generation,
 	t1.generation_expression,
 	col_description(t3.oid, "ordinal_position") AS comment,
@@ -903,10 +943,10 @@ FROM
 	"information_schema"."columns" AS t1
 	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
 	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
-	LEFT JOIN pg_catalog.pg_type AS t4
-	ON t1.udt_name = t4.typname
+	JOIN pg_catalog.pg_attribute AS a ON a.attrelid = t3.oid AND a.attname = t1.column_name
+	LEFT JOIN pg_catalog.pg_type AS t4 ON t1.udt_name = t4.typname AND t4.typnamespace = t2.oid
 WHERE
-	table_schema = $1 AND table_name IN (%s)
+	t1.table_schema = $1 AND t1.table_name IN (%s)
 ORDER BY
 	t1.table_name, t1.ordinal_position
 `
@@ -926,7 +966,7 @@ SELECT
 	pg_index_column_has_property(idx.indexrelid, idx.ord, 'desc') AS desc,
 	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_first') AS nulls_first,
 	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_last') AS nulls_last,
-	obj_description(to_regclass($1 || i.relname)::oid) AS comment,
+	obj_description(i.oid, 'pg_class') AS comment,
 	i.reloptions AS options
 FROM
 	(
