@@ -20,7 +20,7 @@ import (
 type planApply struct{ conn }
 
 // PlanChanges returns a migration plan for the given schema changes.
-func (p *planApply) PlanChanges(ctx context.Context, name string, changes []schema.Change) (*migrate.Plan, error) {
+func (p *planApply) PlanChanges(ctx context.Context, name string, changes []schema.Change, opts ...migrate.PlanOption) (*migrate.Plan, error) {
 	s := &state{
 		conn: p.conn,
 		Plan: migrate.Plan{
@@ -28,6 +28,12 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 			Reversible:    true,
 			Transactional: true,
 		},
+		created: make(map[string]*schema.EnumType),
+		altered: make(map[string]*schema.EnumType),
+		dropped: make(map[string]*schema.EnumType),
+	}
+	for _, o := range opts {
+		o(&s.PlanOptions)
 	}
 	if err := s.plan(ctx, changes); err != nil {
 		return nil, err
@@ -43,8 +49,8 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 // ApplyChanges applies the changes on the database. An error is returned
 // if the driver is unable to produce a plan to do so, or one of the statements
 // is failed or unsupported.
-func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change) error {
-	return sqlx.ApplyChanges(ctx, changes, p)
+func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change, opts ...migrate.PlanOption) error {
+	return sqlx.ApplyChanges(ctx, changes, p, opts...)
 }
 
 // state represents the state of a planning. It is not part of
@@ -53,11 +59,20 @@ func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change) e
 type state struct {
 	conn
 	migrate.Plan
+	migrate.PlanOptions
+	// Track the enums that were created, altered and
+	// dropped, in this phase to avoid duplicate updates.
+	created, altered, dropped map[string]*schema.EnumType
 }
 
 // Exec executes the changes on the database. An error is returned
 // if one of the operations fail, or a change is not supported.
 func (s *state) plan(ctx context.Context, changes []schema.Change) error {
+	if s.SchemaQualifier != nil {
+		if err := sqlx.CheckChangesScope(changes); err != nil {
+			return err
+		}
+	}
 	planned := s.topLevel(changes)
 	planned, err := sqlx.DetachCycles(planned)
 	if err != nil {
@@ -89,7 +104,7 @@ func (s *state) topLevel(changes []schema.Change) []schema.Change {
 	for _, c := range changes {
 		switch c := c.(type) {
 		case *schema.AddSchema:
-			b := Build("CREATE SCHEMA")
+			b := s.Build("CREATE SCHEMA")
 			if sqlx.Has(c.Extra, &schema.IfNotExists{}) {
 				b.P("IF NOT EXISTS")
 			}
@@ -97,11 +112,11 @@ func (s *state) topLevel(changes []schema.Change) []schema.Change {
 			s.append(&migrate.Change{
 				Cmd:     b.String(),
 				Source:  c,
-				Reverse: Build("DROP SCHEMA").Ident(c.S.Name).P("CASCADE").String(),
+				Reverse: s.Build("DROP SCHEMA").Ident(c.S.Name).P("CASCADE").String(),
 				Comment: fmt.Sprintf("Add new schema named %q", c.S.Name),
 			})
 		case *schema.DropSchema:
-			b := Build("DROP SCHEMA")
+			b := s.Build("DROP SCHEMA")
 			if sqlx.Has(c.Extra, &schema.IfExists{}) {
 				b.P("IF EXISTS")
 			}
@@ -121,12 +136,12 @@ func (s *state) topLevel(changes []schema.Change) []schema.Change {
 // addTable builds and executes the query for creating a table in a schema.
 func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 	// Create enum types before using them in the `CREATE TABLE` statement.
-	if err := s.addTypes(ctx, add.T.Columns...); err != nil {
+	if err := s.mayAddEnums(ctx, add.T, add.T.Columns...); err != nil {
 		return err
 	}
 	var (
 		errs []string
-		b    = Build("CREATE TABLE")
+		b    = s.Build("CREATE TABLE")
 	)
 	if sqlx.Has(add.Extra, &schema.IfNotExists{}) {
 		b.P("IF NOT EXISTS")
@@ -134,7 +149,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 	b.Table(add.T)
 	b.Wrap(func(b *sqlx.Builder) {
 		b.MapComma(add.T.Columns, func(i int, b *sqlx.Builder) {
-			if err := s.column(b, add.T.Columns[i]); err != nil {
+			if err := s.column(b, add.T, add.T.Columns[i]); err != nil {
 				errs = append(errs, err.Error())
 			}
 		})
@@ -167,7 +182,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 		Cmd:     b.String(),
 		Source:  add,
 		Comment: fmt.Sprintf("create %q table", add.T.Name),
-		Reverse: Build("DROP TABLE").Table(add.T).String(),
+		Reverse: s.Build("DROP TABLE").Table(add.T).String(),
 	})
 	s.addIndexes(add.T, add.T.Indexes...)
 	s.addComments(add.T)
@@ -176,7 +191,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 
 // dropTable builds and executes the query for dropping a table from a schema.
 func (s *state) dropTable(drop *schema.DropTable) {
-	b := Build("DROP TABLE")
+	b := s.Build("DROP TABLE")
 	if sqlx.Has(drop.Extra, &schema.IfExists{}) {
 		b.P("IF EXISTS")
 	}
@@ -211,7 +226,13 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 			}
 			addI = append(addI, change.I)
 		case *schema.DropIndex:
-			dropI = append(dropI, change.I)
+			// Unlike DROP INDEX statements that are executed separately,
+			// DROP CONSTRAINT are added to the ALTER TABLE statement below.
+			if isUniqueConstraint(change.I) {
+				alter = append(alter, change)
+			} else {
+				dropI = append(dropI, change.I)
+			}
 		case *schema.ModifyIndex:
 			k := change.Change
 			if change.Change.Is(schema.ChangeComment) {
@@ -232,8 +253,8 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 			changes = append(changes, &migrate.Change{
 				Source:  change,
 				Comment: fmt.Sprintf("rename an index from %q to %q", change.From.Name, change.To.Name),
-				Cmd:     Build("ALTER INDEX").Ident(change.From.Name).P("RENAME TO").Ident(change.To.Name).String(),
-				Reverse: Build("ALTER INDEX").Ident(change.To.Name).P("RENAME TO").Ident(change.From.Name).String(),
+				Cmd:     s.Build("ALTER INDEX").Ident(change.From.Name).P("RENAME TO").Ident(change.To.Name).String(),
+				Reverse: s.Build("ALTER INDEX").Ident(change.To.Name).P("RENAME TO").Ident(change.From.Name).String(),
 			})
 		case *schema.ModifyForeignKey:
 			// Foreign-key modification is translated into 2 steps.
@@ -244,7 +265,7 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 				F: change.To,
 			})
 		case *schema.AddColumn:
-			if err := s.addTypes(ctx, change.C); err != nil {
+			if err := s.mayAddEnums(ctx, modify.T, change.C); err != nil {
 				return err
 			}
 			if c := (schema.Comment{}); sqlx.Has(change.C.Attrs, &c) {
@@ -267,9 +288,9 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 			from, ok1 := change.From.Type.Type.(*schema.EnumType)
 			to, ok2 := change.To.Type.Type.(*schema.EnumType)
 			switch {
-			// Enum was changed.
-			case ok1 && ok2 && from.T == to.T:
-				if err := s.alterType(from, to); err != nil {
+			// Enum was changed (underlying values).
+			case ok1 && ok2 && s.enumIdent(modify.T.Schema, from) == s.enumIdent(modify.T.Schema, to):
+				if err := s.alterEnum(modify.T, from, to); err != nil {
 					return err
 				}
 				// If only the enum values were changed,
@@ -277,16 +298,17 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 				if k == schema.ChangeType {
 					continue
 				}
-			// Enum was added (and column type was changed).
-			case !ok1 && ok2:
-				if err := s.addTypes(ctx, change.To); err != nil {
+			// Enum was added or changed.
+			case !ok1 && ok2 ||
+				ok1 && ok2 && s.enumIdent(modify.T.Schema, from) != s.enumIdent(modify.T.Schema, to):
+				if err := s.mayAddEnums(ctx, modify.T, change.To); err != nil {
 					return err
 				}
 			}
-			alter = append(alter, change)
+			alter = append(alter, &schema.ModifyColumn{To: change.To, From: change.From, Change: k})
 		case *schema.RenameColumn:
 			// "RENAME COLUMN" cannot be combined with other alterations.
-			b := Build("ALTER TABLE").Table(modify.T).P("RENAME COLUMN")
+			b := s.Build("ALTER TABLE").Table(modify.T).P("RENAME COLUMN")
 			r := b.Clone()
 			changes = append(changes, &migrate.Change{
 				Source:  change,
@@ -315,18 +337,18 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		reverse    []schema.Change
 		reversible = true
 	)
-	build := func(changes []schema.Change) (string, error) {
-		b := Build("ALTER TABLE").Table(t)
+	build := func(alter *alterChange, changes []schema.Change) (string, error) {
+		b := s.Build("ALTER TABLE").Table(t)
 		err := b.MapCommaErr(changes, func(i int, b *sqlx.Builder) error {
 			switch change := changes[i].(type) {
 			case *schema.AddColumn:
 				b.P("ADD COLUMN")
-				if err := s.column(b, change.C); err != nil {
+				if err := s.column(b, t, change.C); err != nil {
 					return err
 				}
 				reverse = append(reverse, &schema.DropColumn{C: change.C})
 			case *schema.ModifyColumn:
-				if err := s.alterColumn(b, change.Change, change.To); err != nil {
+				if err := s.alterColumn(b, alter, t, change); err != nil {
 					return err
 				}
 				if change.Change.Is(schema.ChangeGenerated) {
@@ -337,9 +359,30 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 					To:     change.From,
 					Change: change.Change & ^schema.ChangeGenerated,
 				})
+				toE, toHas := change.From.Type.Type.(*schema.EnumType)
+				fromE, fromHas := change.From.Type.Type.(*schema.EnumType)
+				// In case the enum was dropped or replaced with a different one.
+				if fromHas && !toHas || fromHas && toHas && s.enumIdent(t.Schema, fromE) != s.enumIdent(t.Schema, toE) {
+					if err := s.mayDropEnum(alter, t.Schema, fromE); err != nil {
+						return err
+					}
+				}
 			case *schema.DropColumn:
 				b.P("DROP COLUMN").Ident(change.C.Name)
 				reverse = append(reverse, &schema.AddColumn{C: change.C})
+				if e, ok := change.C.Type.Type.(*schema.EnumType); ok {
+					if err := s.mayDropEnum(alter, t.Schema, e); err != nil {
+						return err
+					}
+				}
+			case *schema.AddIndex:
+				b.P("ADD CONSTRAINT").Ident(change.I.Name).P("UNIQUE")
+				s.indexParts(b, change.I.Parts)
+				// Skip reversing this operation as it is the inverse of
+				// the operation below and should not be used besides this.
+			case *schema.DropIndex:
+				b.P("DROP CONSTRAINT").Ident(change.I.Name)
+				reverse = append(reverse, &schema.AddIndex{I: change.I})
 			case *schema.AddForeignKey:
 				b.P("ADD")
 				s.fks(b, change.F)
@@ -383,12 +426,13 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		}
 		return b.String(), nil
 	}
-	cmd, err := build(changes)
+	cmd := &alterChange{}
+	stmt, err := build(cmd, changes)
 	if err != nil {
 		return fmt.Errorf("alter table %q: %v", t.Name, err)
 	}
-	change := &migrate.Change{
-		Cmd: cmd,
+	cmd.main = &migrate.Change{
+		Cmd: stmt,
 		Source: &schema.ModifyTable{
 			T:       t,
 			Changes: changes,
@@ -399,11 +443,166 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		// Changes should be reverted in
 		// a reversed order they were created.
 		sqlx.ReverseChanges(reverse)
-		if change.Reverse, err = build(reverse); err != nil {
+		if cmd.main.Reverse, err = build(&alterChange{}, reverse); err != nil {
 			return fmt.Errorf("reverse alter table %q: %v", t.Name, err)
 		}
 	}
-	s.append(change)
+	cmd.append(s)
+	return nil
+}
+
+// alterChange describes an alter table migrate.Change where its main command
+// can be supported by additional statements before and after it is executed.
+type alterChange struct {
+	main          *migrate.Change
+	before, after []*migrate.Change
+}
+
+func (a *alterChange) append(s *state) {
+	s.append(a.before...)
+	s.append(a.main)
+	s.append(a.after...)
+}
+
+func (s *state) alterColumn(b *sqlx.Builder, alter *alterChange, t *schema.Table, c *schema.ModifyColumn) error {
+	for k := c.Change; !k.Is(schema.NoChange); {
+		b.P("ALTER COLUMN").Ident(c.To.Name)
+		switch {
+		case k.Is(schema.ChangeType):
+			if err := s.alterType(b, alter, t, c); err != nil {
+				return err
+			}
+			k &= ^schema.ChangeType
+		case k.Is(schema.ChangeNull) && c.To.Type.Null:
+			if t, ok := c.To.Type.Type.(*SerialType); ok {
+				return fmt.Errorf("NOT NULL constraint is required for %s column %q", t.T, c.To.Name)
+			}
+			b.P("DROP NOT NULL")
+			k &= ^schema.ChangeNull
+		case k.Is(schema.ChangeNull) && !c.To.Type.Null:
+			b.P("SET NOT NULL")
+			k &= ^schema.ChangeNull
+		case k.Is(schema.ChangeDefault) && c.To.Default == nil:
+			b.P("DROP DEFAULT")
+			k &= ^schema.ChangeDefault
+		case k.Is(schema.ChangeDefault) && c.To.Default != nil:
+			s.columnDefault(b.P("SET"), c.To)
+			k &= ^schema.ChangeDefault
+		case k.Is(schema.ChangeAttr):
+			toI, ok := identity(c.To.Attrs)
+			if !ok {
+				return fmt.Errorf("unexpected attribute change (expect IDENTITY): %v", c.To.Attrs)
+			}
+			// The syntax for altering identity columns is identical to sequence_options.
+			// https://www.postgresql.org/docs/current/sql-altersequence.html
+			b.P("SET GENERATED", toI.Generation, "SET START WITH", strconv.FormatInt(toI.Sequence.Start, 10), "SET INCREMENT BY", strconv.FormatInt(toI.Sequence.Increment, 10))
+			// Skip SEQUENCE RESTART in case the "start value" is less than the "current value" in one
+			// of the states (inspected and desired), because this function is used for both UP and DOWN.
+			if fromI, ok := identity(c.From.Attrs); (!ok || fromI.Sequence.Last < toI.Sequence.Start) && toI.Sequence.Last < toI.Sequence.Start {
+				b.P("RESTART")
+			}
+			k &= ^schema.ChangeAttr
+		case k.Is(schema.ChangeGenerated):
+			if sqlx.Has(c.To.Attrs, &schema.GeneratedExpr{}) {
+				return fmt.Errorf("unexpected generation expression change (expect DROP EXPRESSION): %v", c.To.Attrs)
+			}
+			b.P("DROP EXPRESSION")
+			k &= ^schema.ChangeGenerated
+		default: // e.g. schema.ChangeComment.
+			return fmt.Errorf("unexpected column change: %d", k)
+		}
+		if !k.Is(schema.NoChange) {
+			b.Comma()
+		}
+	}
+	return nil
+}
+
+// alterType appends the clause(s) to alter the column type and assuming the
+// "ALTER COLUMN <Name>" was called before by the alterColumn function.
+func (s *state) alterType(b *sqlx.Builder, alter *alterChange, t *schema.Table, c *schema.ModifyColumn) error {
+	// Commands for creating and dropping serial sequences.
+	createDropSeq := func(st *SerialType) (string, string, string) {
+		seq := fmt.Sprintf(`%s%q`, s.schemaPrefix(t.Schema), st.sequence(t, c.To))
+		drop := s.Build("DROP SEQUENCE IF EXISTS").P(seq).String()
+		create := s.Build("CREATE SEQUENCE IF NOT EXISTS").P(seq, "OWNED BY").
+			P(fmt.Sprintf(`%s%q.%q`, s.schemaPrefix(t.Schema), t.Name, c.To.Name)).
+			String()
+		return create, drop, seq
+	}
+	toS, toHas := c.To.Type.Type.(*SerialType)
+	fromS, fromHas := c.From.Type.Type.(*SerialType)
+	switch {
+	// Sequence was dropped.
+	case fromHas && !toHas:
+		b.P("DROP DEFAULT")
+		create, drop, _ := createDropSeq(fromS)
+		// Sequence should be deleted after it was dropped
+		// from the DEFAULT value.
+		alter.after = append(alter.after, &migrate.Change{
+			Source:  c,
+			Comment: fmt.Sprintf("drop sequence used by serial column %q", c.From.Name),
+			Cmd:     drop,
+			Reverse: create,
+		})
+		toT, err := FormatType(c.To.Type.Type)
+		if err != nil {
+			return err
+		}
+		fromT, err := FormatType(fromS.IntegerType())
+		if err != nil {
+			return err
+		}
+		// Underlying type was changed. e.g. serial to bigint.
+		if toT != fromT {
+			b.Comma().P("ALTER COLUMN").Ident(c.To.Name).P("TYPE", toT)
+		}
+	// Sequence was added.
+	case !fromHas && toHas:
+		create, drop, seq := createDropSeq(toS)
+		// Sequence should be created before it is used by the
+		// column DEFAULT value.
+		alter.before = append(alter.before, &migrate.Change{
+			Source:  c,
+			Comment: fmt.Sprintf("create sequence for serial column %q", c.To.Name),
+			Cmd:     create,
+			Reverse: drop,
+		})
+		b.P("SET DEFAULT", fmt.Sprintf("nextval('%s')", seq))
+		toT, err := FormatType(toS.IntegerType())
+		if err != nil {
+			return err
+		}
+		fromT, err := FormatType(c.From.Type.Type)
+		if err != nil {
+			return err
+		}
+		// Underlying type was changed. e.g. integer to bigserial (bigint).
+		if toT != fromT {
+			b.Comma().P("ALTER COLUMN").Ident(c.To.Name).P("TYPE", toT)
+		}
+	// Serial type was changed. e.g. serial to bigserial.
+	case fromHas && toHas:
+		f, err := FormatType(toS.IntegerType())
+		if err != nil {
+			return err
+		}
+		b.P("TYPE", f)
+	default:
+		var (
+			f   string
+			err error
+		)
+		if e, ok := c.To.Type.Type.(*schema.EnumType); ok {
+			f = s.enumIdent(t.Schema, e)
+		} else if f, err = FormatType(c.To.Type.Type); err != nil {
+			return err
+		}
+		b.P("TYPE", f)
+	}
+	if collate := (schema.Collation{}); sqlx.Has(c.To.Attrs, &collate) {
+		b.P("COLLATE", collate.V)
+	}
 	return nil
 }
 
@@ -411,8 +610,8 @@ func (s *state) renameTable(c *schema.RenameTable) {
 	s.append(&migrate.Change{
 		Source:  c,
 		Comment: fmt.Sprintf("rename a table from %q to %q", c.From.Name, c.To.Name),
-		Cmd:     Build("ALTER TABLE").Table(c.From).P("RENAME TO").Table(c.To).String(),
-		Reverse: Build("ALTER TABLE").Table(c.To).P("RENAME TO").Table(c.From).String(),
+		Cmd:     s.Build("ALTER TABLE").Table(c.From).P("RENAME TO").Table(c.To).String(),
+		Reverse: s.Build("ALTER TABLE").Table(c.To).P("RENAME TO").Table(c.From).String(),
 	})
 }
 
@@ -433,8 +632,8 @@ func (s *state) addComments(t *schema.Table) {
 	}
 }
 
-func (*state) tableComment(t *schema.Table, to, from string) *migrate.Change {
-	b := Build("COMMENT ON TABLE").Table(t).P("IS")
+func (s *state) tableComment(t *schema.Table, to, from string) *migrate.Change {
+	b := s.Build("COMMENT ON TABLE").Table(t).P("IS")
 	return &migrate.Change{
 		Cmd:     b.Clone().P(quote(to)).String(),
 		Comment: fmt.Sprintf("set comment to table: %q", t.Name),
@@ -442,8 +641,8 @@ func (*state) tableComment(t *schema.Table, to, from string) *migrate.Change {
 	}
 }
 
-func (*state) columnComment(t *schema.Table, c *schema.Column, to, from string) *migrate.Change {
-	b := Build("COMMENT ON COLUMN").Table(t)
+func (s *state) columnComment(t *schema.Table, c *schema.Column, to, from string) *migrate.Change {
+	b := s.Build("COMMENT ON COLUMN").Table(t)
 	b.WriteByte('.')
 	b.Ident(c.Name).P("IS")
 	return &migrate.Change{
@@ -453,8 +652,8 @@ func (*state) columnComment(t *schema.Table, c *schema.Column, to, from string)
 	}
 }
 
-func (*state) indexComment(t *schema.Table, idx *schema.Index, to, from string) *migrate.Change {
-	b := Build("COMMENT ON INDEX").Ident(idx.Name).P("IS")
+func (s *state) indexComment(t *schema.Table, idx *schema.Index, to, from string) *migrate.Change {
+	b := s.Build("COMMENT ON INDEX").Ident(idx.Name).P("IS")
 	return &migrate.Change{
 		Cmd:     b.Clone().P(quote(to)).String(),
 		Comment: fmt.Sprintf("set comment to index: %q on table: %q", idx.Name, t.Name),
@@ -474,7 +673,7 @@ func (s *state) dropIndexes(t *schema.Table, indexes ...*schema.Index) {
 	}
 }
 
-func (s *state) addTypes(ctx context.Context, columns ...*schema.Column) error {
+func (s *state) mayAddEnums(ctx context.Context, t *schema.Table, columns ...*schema.Column) error {
 	for _, c := range columns {
 		e, ok := c.Type.Type.(*schema.EnumType)
 		if !ok {
@@ -483,28 +682,32 @@ func (s *state) addTypes(ctx context.Context, columns ...*schema.Column) error {
 		if e.T == "" {
 			return fmt.Errorf("missing enum name for column %q", c.Name)
 		}
-		c.Type.Raw = e.T
-		if exists, err := s.enumExists(ctx, e.T); err != nil {
+		if exists, err := s.enumExists(ctx, t.Schema, e); err != nil {
 			return err
 		} else if exists {
+			// Enum exists and was not created
+			// on this migration phase.
 			continue
 		}
-		b := Build("CREATE TYPE").Ident(e.T).P("AS ENUM")
-		b.Wrap(func(b *sqlx.Builder) {
-			b.MapComma(e.Values, func(i int, b *sqlx.Builder) {
-				b.WriteString("'" + e.Values[i] + "'")
-			})
-		})
+		name := s.enumIdent(t.Schema, e)
+		if prev, ok := s.created[name]; ok {
+			if !sqlx.ValuesEqual(prev.Values, e.Values) {
+				return fmt.Errorf("enum type %s has inconsistent desired state: %q != %q", name, prev.Values, e.Values)
+			}
+			continue
+		}
+		s.created[name] = e
+		create, drop := s.createDropEnum(t.Schema, e)
 		s.append(&migrate.Change{
-			Cmd:     b.String(),
+			Cmd:     create,
+			Reverse: drop,
 			Comment: fmt.Sprintf("create enum type %q", e.T),
-			Reverse: Build("DROP TYPE").Ident(e.T).String(),
 		})
 	}
 	return nil
 }
 
-func (s *state) alterType(from, to *schema.EnumType) error {
+func (s *state) alterEnum(t *schema.Table, from, to *schema.EnumType) error {
 	if len(from.Values) > len(to.Values) {
 		return fmt.Errorf("dropping enum (%q) value is not supported", from.T)
 	}
@@ -513,27 +716,73 @@ func (s *state) alterType(from, to *schema.EnumType) error {
 			return fmt.Errorf("replacing or reordering enum (%q) value is not supported: %q != %q", to.T, to.Values, from.Values)
 		}
 	}
+	name := s.enumIdent(t.Schema, from)
+	if prev, ok := s.altered[name]; ok {
+		if !sqlx.ValuesEqual(prev.Values, to.Values) {
+			return fmt.Errorf("enum type %s has inconsistent desired state: %q != %q", name, prev.Values, to.Values)
+		}
+		return nil
+	}
+	s.altered[name] = to
 	for _, v := range to.Values[len(from.Values):] {
 		s.append(&migrate.Change{
-			Cmd:     Build("ALTER TYPE").Ident(from.T).P("ADD VALUE", quote(v)).String(),
+			Cmd:     s.Build("ALTER TYPE").P(name, "ADD VALUE", quote(v)).String(),
 			Comment: fmt.Sprintf("add value to enum type: %q", from.T),
 		})
 	}
 	return nil
 }
 
-func (s *state) enumExists(ctx context.Context, name string) (bool, error) {
-	rows, err := s.QueryContext(ctx, "SELECT * FROM pg_type WHERE typname = $1 AND typtype = 'e'", name)
+func (s *state) enumExists(ctx context.Context, ns *schema.Schema, e *schema.EnumType) (bool, error) {
+	query, args := `SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e'`, []any{e.T}
+	if es := s.enumSchema(ns, e); es != "" {
+		query += " AND n.nspname = $2"
+		args = append(args, es)
+	}
+	rows, err := s.QueryContext(ctx, query, args...)
 	if err != nil {
-		return false, fmt.Errorf("check index existence: %w", err)
+		return false, fmt.Errorf("check enum existence: %w", err)
 	}
 	defer rows.Close()
 	return rows.Next(), rows.Err()
 }
 
+// mayDropEnum drops dangling enum types form the schema.
+func (s *state) mayDropEnum(alter *alterChange, ns *schema.Schema, e *schema.EnumType) error {
+	name := s.enumIdent(ns, e)
+	if _, ok := s.dropped[name]; ok {
+		return nil
+	}
+	schemas := []*schema.Schema{ns}
+	// In case there is a realm attached, traverse the entire tree.
+	if ns.Realm != nil && len(ns.Realm.Schemas) > 0 {
+		schemas = ns.Realm.Schemas
+	}
+	for i := range schemas {
+		for _, t := range schemas[i].Tables {
+			for _, c := range t.Columns {
+				e1, ok := c.Type.Type.(*schema.EnumType)
+				// Although we search in siblings schemas, use the
+				// table's one for building the enum identifier.
+				if ok && s.enumIdent(ns, e1) == name {
+					return nil
+				}
+			}
+		}
+	}
+	s.dropped[name] = e
+	create, drop := s.createDropEnum(ns, e)
+	alter.after = append(alter.after, &migrate.Change{
+		Cmd:     drop,
+		Reverse: create,
+		Comment: fmt.Sprintf("drop enum type %q", e.T),
+	})
+	return nil
+}
+
 func (s *state) addIndexes(t *schema.Table, indexes ...*schema.Index) {
 	for _, idx := range indexes {
-		b := Build("CREATE")
+		b := s.Build("CREATE")
 		if idx.Unique {
 			b.P("UNIQUE")
 		}
@@ -547,16 +796,13 @@ func (s *state) addIndexes(t *schema.Table, indexes ...*schema.Index) {
 			Cmd:     b.String(),
 			Comment: fmt.Sprintf("create index %q to table: %q", idx.Name, t.Name),
 			Reverse: func() string {
-				b := Build("DROP INDEX")
+				b := s.Build("DROP INDEX")
 				// Unlike MySQL, the DROP command is not attached to ALTER TABLE.
 				// Therefore, we print indexes with their qualified name, because
 				// the connection that executes the statements may not be attached
 				// to this schema.
 				if t.Schema != nil {
-					b.WriteByte(b.QuoteChar)
-					b.WriteString(t.Schema.Name)
-					b.WriteByte(b.QuoteChar)
-					b.WriteByte('.')
+					b.WriteString(s.schemaPrefix(t.Schema))
 				}
 				b.Ident(idx.Name)
 				return b.String()
@@ -565,14 +811,21 @@ func (s *state) addIndexes(t *schema.Table, indexes ...*schema.Index) {
 	}
 }
 
-func (s *state) column(b *sqlx.Builder, c *schema.Column) error {
-	t, err := FormatType(c.Type.Type)
-	if err != nil {
+func (s *state) column(b *sqlx.Builder, t *schema.Table, c *schema.Column) error {
+	var (
+		f   string
+		err error
+	)
+	if e, ok := c.Type.Type.(*schema.EnumType); ok {
+		f = s.enumIdent(t.Schema, e)
+	} else if f, err = FormatType(c.Type.Type); err != nil {
 		return err
 	}
-	b.Ident(c.Name).P(t)
+	b.Ident(c.Name).P(f)
 	if !c.Type.Null {
 		b.P("NOT")
+	} else if t, ok := c.Type.Type.(*SerialType); ok {
+		return fmt.Errorf("NOT NULL constraint is required for %s column %q", t.T, c.Name)
 	}
 	b.P("NULL")
 	s.columnDefault(b, c)
@@ -630,60 +883,6 @@ func (s *state) columnDefault(b *sqlx.Builder, c *schema.Column) {
 	}
 }
 
-func (s *state) alterColumn(b *sqlx.Builder, k schema.ChangeKind, c *schema.Column) error {
-	for !k.Is(schema.NoChange) {
-		b.P("ALTER COLUMN").Ident(c.Name)
-		switch {
-		case k.Is(schema.ChangeType):
-			t, err := FormatType(c.Type.Type)
-			if err != nil {
-				return err
-			}
-			b.P("TYPE", t)
-			if collate := (schema.Collation{}); sqlx.Has(c.Attrs, &collate) {
-				b.P("COLLATE", collate.V)
-			}
-			k &= ^schema.ChangeType
-		case k.Is(schema.ChangeNull) && c.Type.Null:
-			b.P("DROP NOT NULL")
-			k &= ^schema.ChangeNull
-		case k.Is(schema.ChangeNull) && !c.Type.Null:
-			b.P("SET NOT NULL")
-			k &= ^schema.ChangeNull
-		case k.Is(schema.ChangeDefault) && c.Default == nil:
-			b.P("DROP DEFAULT")
-			k &= ^schema.ChangeDefault
-		case k.Is(schema.ChangeDefault) && c.Default != nil:
-			s.columnDefault(b.P("SET"), c)
-			k &= ^schema.ChangeDefault
-		case k.Is(schema.ChangeAttr):
-			id, ok := identity(c.Attrs)
-			if !ok {
-				return fmt.Errorf("unexpected attribute change (expect IDENTITY): %v", c.Attrs)
-			}
-			// The syntax for altering identity columns is identical to sequence_options.
-			// https://www.postgresql.org/docs/current/sql-altersequence.html
-			b.P("SET GENERATED", id.Generation, "SET START WITH", strconv.FormatInt(id.Sequence.Start, 10), "SET INCREMENT BY", strconv.FormatInt(id.Sequence.Increment, 10), "RESTART")
-			k &= ^schema.ChangeAttr
-		case k.Is(schema.ChangeGenerated):
-			if sqlx.Has(c.Attrs, &schema.GeneratedExpr{}) {
-				return fmt.Errorf("unexpected generation expression change (expect DROP EXPRESSION): %v", c.Attrs)
-			}
-			b.P("DROP EXPRESSION")
-			k &= ^schema.ChangeGenerated
-		case k.Is(schema.ChangeComment):
-			// Handled separately on modifyTable.
-			k &= ^schema.ChangeComment
-		default:
-			return fmt.Errorf("unexpected column change: %d", k)
-		}
-		if !k.Is(schema.NoChange) {
-			b.Comma()
-		}
-	}
-	return nil
-}
-
 func (s *state) indexParts(b *sqlx.Builder, parts []*schema.IndexPart) {
 	b.Wrap(func(b *sqlx.Builder) {
 		b.MapComma(parts, func(i int, b *sqlx.Builder) {
@@ -786,9 +985,9 @@ func (s *state) append(c ...*migrate.Change) {
 }
 
 // Build instantiates a new builder and writes the given phrase to it.
-func Build(phrase string) *sqlx.Builder {
-	b := &sqlx.Builder{QuoteChar: '"'}
-	return b.P(phrase)
+func (s *state) Build(phrases ...string) *sqlx.Builder {
+	b := &sqlx.Builder{QuoteChar: '"', Schema: s.SchemaQualifier}
+	return b.P(phrases...)
 }
 
 // skipAutoChanges filters unnecessary changes that are automatically
@@ -807,7 +1006,7 @@ search:
 	for _, c := range changes {
 		switch c := c.(type) {
 		// Indexes involving the column are automatically dropped
-		// with it. This true for multi-columns indexes as well.
+		// with it. This is true for multi-columns indexes as well.
 		// See https://www.postgresql.org/docs/current/sql-altertable.html
 		case *schema.DropIndex:
 			for _, p := range c.I.Parts {
@@ -864,9 +1063,86 @@ func check(b *sqlx.Builder, c *schema.Check) {
 	}
 }
 
+// isUniqueConstraint reports if the index is a valid UNIQUE constraint.
+func isUniqueConstraint(i *schema.Index) bool {
+	if c := (ConType{}); !sqlx.Has(i.Attrs, &c) || !c.IsUnique() || !i.Unique {
+		return false
+	}
+	// UNIQUE constraint cannot use functional indexes,
+	// and all its parts must have the default sort ordering.
+	for _, p := range i.Parts {
+		if p.X != nil || p.Desc {
+			return false
+		}
+	}
+	for _, a := range i.Attrs {
+		switch a := a.(type) {
+		// UNIQUE constraints must have BTREE type indexes.
+		case *IndexType:
+			if strings.ToUpper(a.T) != IndexTypeBTree {
+				return false
+			}
+		// Partial indexes are not allowed.
+		case *IndexPredicate:
+			return false
+		}
+	}
+	return true
+}
+
 func quote(s string) string {
 	if sqlx.IsQuoted(s, '\'') {
 		return s
 	}
 	return "'" + strings.ReplaceAll(s, "'", "''") + "'"
 }
+
+func (s *state) createDropEnum(sc *schema.Schema, e *schema.EnumType) (string, string) {
+	name := s.enumIdent(sc, e)
+	return s.Build("CREATE TYPE").
+			P(name, "AS ENUM").
+			Wrap(func(b *sqlx.Builder) {
+				b.MapComma(e.Values, func(i int, b *sqlx.Builder) {
+					b.WriteString(quote(e.Values[i]))
+				})
+			}).
+			String(),
+		s.Build("DROP TYPE").P(name).String()
+}
+
+func (s *state) enumIdent(ns *schema.Schema, e *schema.EnumType) string {
+	es := s.enumSchema(ns, e)
+	if es != "" {
+		return fmt.Sprintf("%q.%q", es, e.T)
+	}
+	return strconv.Quote(e.T)
+}
+
+func (s *state) enumSchema(ns *schema.Schema, e *schema.EnumType) (es string) {
+	switch {
+	// In case the plan uses a specific schema qualifier.
+	case s.SchemaQualifier != nil:
+		es = *s.SchemaQualifier
+	// Enum schema has higher precedence.
+	case e.Schema != nil:
+		es = e.Schema.Name
+	// Fallback to table schema if exists.
+	case ns != nil:
+		es = ns.Name
+	}
+	return
+}
+
+// schemaPrefix returns the schema prefix based on the planner config.
+func (s *state) schemaPrefix(ns *schema.Schema) string {
+	switch {
+	case s.SchemaQualifier != nil:
+		// In case the qualifier is empty, ignore.
+		if *s.SchemaQualifier != "" {
+			return fmt.Sprintf("%q.", *s.SchemaQualifier)
+		}
+	case ns != nil && ns.Name != "":
+		return fmt.Sprintf("%q.", ns.Name)
+	}
+	return ""
+}
