@@ -19,9 +19,11 @@ import (
 
 func TestPlanChanges(t *testing.T) {
 	tests := []struct {
-		changes []schema.Change
-		mock    func(mock)
-		plan    *migrate.Plan
+		changes  []schema.Change
+		options  []migrate.PlanOption
+		mock     func(mock)
+		wantPlan *migrate.Plan
+		wantErr  bool
 	}{
 		{
 			changes: []schema.Change{
@@ -29,7 +31,7 @@ func TestPlanChanges(t *testing.T) {
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{&schema.IfExists{}}},
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{}},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -86,7 +88,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -104,14 +106,7 @@ func TestPlanChanges(t *testing.T) {
 						Name: "users",
 						Columns: []*schema.Column{
 							{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}},
-							{
-								Name: "name",
-								Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}},
-								Indexes: []*schema.Index{
-									schema.NewIndex("name_index").
-										AddParts(schema.NewColumnPart(schema.NewColumn("name"))),
-								},
-							}},
+							{Name: "name", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}}}},
 					}
 					return &schema.ModifyTable{
 						T: users,
@@ -124,7 +119,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -135,11 +130,43 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
+		{
+			changes: []schema.Change{
+				func() schema.Change {
+					users := &schema.Table{
+						Name: "users",
+						Columns: []*schema.Column{
+							{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}},
+							{Name: "nickname", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}}}},
+					}
+					return &schema.ModifyTable{
+						T: users,
+						Changes: []schema.Change{
+							&schema.DropIndex{
+								I: schema.NewUniqueIndex("unique_nickname").
+									AddColumns(schema.NewColumn("nickname")).
+									AddAttrs(&ConType{T: "u"}),
+							},
+						},
+					}
+				}(),
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `ALTER TABLE "users" DROP CONSTRAINT "unique_nickname"`,
+						Reverse: `ALTER TABLE "users" ADD CONSTRAINT "unique_nickname" UNIQUE ("nickname")`,
+					},
+				},
+			},
+		},
 		{
 			changes: []schema.Change{
 				&schema.AddSchema{S: &schema.Schema{Name: "test"}},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `CREATE SCHEMA "test"`, Reverse: `DROP SCHEMA "test" CASCADE`}}},
@@ -148,7 +175,7 @@ func TestPlanChanges(t *testing.T) {
 			changes: []schema.Change{
 				&schema.DropSchema{S: &schema.Schema{Name: "atlas"}},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `DROP SCHEMA "atlas" CASCADE`}},
 			},
@@ -172,7 +199,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `CREATE TABLE "posts" ("id" integer NOT NULL GENERATED BY DEFAULT AS IDENTITY, "text" text NULL, CONSTRAINT "id_nonzero" CHECK ("id" > 0), CONSTRAINT "text_len" CHECK (length("text") > 0) NO INHERIT, CONSTRAINT "a_in_b" CHECK ((a) in (b))) PARTITION BY HASH ("text")`, Reverse: `DROP TABLE "posts"`}},
@@ -189,7 +216,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `CREATE TABLE "posts" ("id" integer NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 1024))`, Reverse: `DROP TABLE "posts"`}},
@@ -207,7 +234,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -236,7 +263,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -257,7 +284,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `CREATE TABLE "posts" ("id" integer NOT NULL GENERATED BY DEFAULT AS IDENTITY (INCREMENT BY 2))`, Reverse: `DROP TABLE "posts"`}},
@@ -274,7 +301,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes:       []*migrate.Change{{Cmd: `CREATE TABLE "posts" ("id" integer NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 100 INCREMENT BY 2))`, Reverse: `DROP TABLE "posts"`}},
@@ -284,7 +311,7 @@ func TestPlanChanges(t *testing.T) {
 			changes: []schema.Change{
 				&schema.DropTable{T: &schema.Table{Name: "posts"}},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -348,7 +375,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -395,7 +422,46 @@ func TestPlanChanges(t *testing.T) {
 								C: &schema.Column{Name: "name", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar"}}},
 							},
 							&schema.ModifyColumn{
-								From:   &schema.Column{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}, Attrs: []schema.Attr{&Identity{}}},
+								From:   &schema.Column{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}, Attrs: []schema.Attr{&Identity{}, &schema.Comment{Text: "comment"}}},
+								To:     &schema.Column{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}, Attrs: []schema.Attr{&Identity{Sequence: &Sequence{Start: 1024}}}},
+								Change: schema.ChangeAttr | schema.ChangeComment,
+							},
+						},
+					}
+				}(),
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `ALTER TABLE "users" DROP COLUMN "name", ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1024 SET INCREMENT BY 1 RESTART`,
+						Reverse: `ALTER TABLE "users" ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1 SET INCREMENT BY 1 RESTART, ADD COLUMN "name" character varying NOT NULL`,
+					},
+					{
+						Cmd:     `COMMENT ON COLUMN "users" ."id" IS ''`,
+						Reverse: `COMMENT ON COLUMN "users" ."id" IS 'comment'`,
+					},
+				},
+			},
+		},
+		{
+			changes: []schema.Change{
+				func() schema.Change {
+					users := &schema.Table{
+						Name: "users",
+						Columns: []*schema.Column{
+							{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}},
+						},
+					}
+					return &schema.ModifyTable{
+						T: users,
+						Changes: []schema.Change{
+							&schema.DropColumn{
+								C: &schema.Column{Name: "name", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar"}}},
+							},
+							&schema.ModifyColumn{
+								From:   &schema.Column{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}, Attrs: []schema.Attr{&Identity{Sequence: &Sequence{Start: 0, Last: 1025}}}},
 								To:     &schema.Column{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}, Attrs: []schema.Attr{&Identity{Sequence: &Sequence{Start: 1024}}}},
 								Change: schema.ChangeAttr,
 							},
@@ -403,13 +469,13 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
 					{
-						Cmd:     `ALTER TABLE "users" DROP COLUMN "name", ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1024 SET INCREMENT BY 1 RESTART`,
-						Reverse: `ALTER TABLE "users" ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1 SET INCREMENT BY 1 RESTART, ADD COLUMN "name" character varying NOT NULL`,
+						Cmd:     `ALTER TABLE "users" DROP COLUMN "name", ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1024 SET INCREMENT BY 1`,
+						Reverse: `ALTER TABLE "users" ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1 SET INCREMENT BY 1, ADD COLUMN "name" character varying NOT NULL`,
 					},
 				},
 			},
@@ -425,7 +491,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -445,7 +511,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -456,51 +522,67 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				func() schema.Change {
-					users := &schema.Table{
-						Name: "users",
-						Columns: []*schema.Column{
-							{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
-						},
-					}
+					users := schema.NewTable("users").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("a", "b"), schema.EnumSchema(schema.New("test"))),
+						)
 					return &schema.ModifyTable{
 						T: users,
 						Changes: []schema.Change{
 							&schema.ModifyColumn{
 								From:   &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.StringType{T: "text"}}},
-								To:     &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
+								To:     users.Columns[0],
 								Change: schema.ChangeType,
 							},
+							&schema.ModifyColumn{
+								From:   &schema.Column{Name: "status", Type: &schema.ColumnType{Type: &schema.StringType{T: "text"}}},
+								To:     users.Columns[1],
+								Change: schema.ChangeType,
+							},
+							&schema.DropColumn{
+								C: schema.NewEnumColumn("dc1", schema.EnumName("de"), schema.EnumValues("on")),
+							},
+							&schema.DropColumn{
+								C: schema.NewEnumColumn("dc2", schema.EnumName("de"), schema.EnumValues("on")),
+							},
 						},
 					}
 				}(),
 			},
 			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type WHERE typname = $1 AND typtype = 'e'")).
-					WithArgs("state").WillReturnRows(sqlmock.NewRows([]string{"name"}))
+				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
+					WithArgs("state", "public").
+					WillReturnRows(sqlmock.NewRows([]string{"name"}))
+				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
+					WithArgs("status", "test").
+					WillReturnRows(sqlmock.NewRows([]string{"name"}))
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
-					{Cmd: `CREATE TYPE "state" AS ENUM ('on', 'off')`, Reverse: `DROP TYPE "state"`},
-					{Cmd: `ALTER TABLE "users" ALTER COLUMN "state" TYPE state`, Reverse: `ALTER TABLE "users" ALTER COLUMN "state" TYPE text`},
+					{Cmd: `CREATE TYPE "public"."state" AS ENUM ('on', 'off')`, Reverse: `DROP TYPE "public"."state"`},
+					{Cmd: `CREATE TYPE "test"."status" AS ENUM ('a', 'b')`, Reverse: `DROP TYPE "test"."status"`},
+					{Cmd: `ALTER TABLE "public"."users" ALTER COLUMN "state" TYPE "public"."state", ALTER COLUMN "status" TYPE "test"."status", DROP COLUMN "dc1", DROP COLUMN "dc2"`, Reverse: `ALTER TABLE "public"."users" ADD COLUMN "dc2" "public"."de" NOT NULL, ADD COLUMN "dc1" "public"."de" NOT NULL, ALTER COLUMN "status" TYPE text, ALTER COLUMN "state" TYPE text`},
+					{Cmd: `DROP TYPE "public"."de"`, Reverse: `CREATE TYPE "public"."de" AS ENUM ('on')`},
 				},
 			},
 		},
 		{
 			changes: []schema.Change{
 				func() schema.Change {
-					users := &schema.Table{
-						Name: "users",
-						Columns: []*schema.Column{
-							{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
-						},
-					}
+					users := schema.NewTable("users").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+						)
 					return &schema.ModifyTable{
 						T: users,
 						Changes: []schema.Change{
 							&schema.ModifyColumn{
-								From:   &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
+								From:   users.Columns[0],
 								To:     &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off", "unknown"}}}},
 								Change: schema.ChangeType,
 							},
@@ -508,11 +590,11 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
-					{Cmd: `ALTER TYPE "state" ADD VALUE 'unknown'`},
+					{Cmd: `ALTER TYPE "public"."state" ADD VALUE 'unknown'`},
 				},
 			},
 		},
@@ -520,12 +602,11 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				func() schema.Change {
-					users := &schema.Table{
-						Name: "users",
-						Columns: []*schema.Column{
-							{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
-						},
-					}
+					users := schema.NewTable("users").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+						)
 					return &schema.ModifyTable{
 						T: users,
 						Changes: []schema.Change{
@@ -538,12 +619,12 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
-					{Cmd: `ALTER TYPE "state" ADD VALUE 'unknown'`},
-					{Cmd: `COMMENT ON COLUMN "users" ."state" IS ''`, Reverse: `COMMENT ON COLUMN "users" ."state" IS 'foo'`},
+					{Cmd: `ALTER TYPE "public"."state" ADD VALUE 'unknown'`},
+					{Cmd: `COMMENT ON COLUMN "public"."users" ."state" IS ''`, Reverse: `COMMENT ON COLUMN "public"."users" ."state" IS 'foo'`},
 				},
 			},
 		},
@@ -569,7 +650,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -600,7 +681,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -634,7 +715,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -676,7 +757,7 @@ func TestPlanChanges(t *testing.T) {
 					}
 				}(),
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -694,7 +775,7 @@ func TestPlanChanges(t *testing.T) {
 					To:   schema.NewTable("t2").SetSchema(schema.New("s2")),
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -717,7 +798,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -743,7 +824,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -770,7 +851,7 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
-			plan: &migrate.Plan{
+			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
@@ -781,6 +862,252 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
+		// Invalid serial type.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{
+					T: &schema.Table{
+						Name: "posts",
+						Columns: []*schema.Column{
+							{Name: "id", Type: &schema.ColumnType{Type: &SerialType{T: "serial"}, Null: true}},
+						},
+					},
+				},
+			},
+			wantErr: true,
+		},
+		// Drop serial sequence.
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewIntColumn("c1", "integer"),
+							schema.NewIntColumn("c2", "integer"),
+						),
+					Changes: schema.Changes{
+						&schema.ModifyColumn{
+							From:   schema.NewColumn("c1").SetType(&SerialType{T: "smallserial"}),
+							To:     schema.NewIntColumn("c1", "integer"),
+							Change: schema.ChangeType,
+						},
+						&schema.ModifyColumn{
+							From:   schema.NewColumn("c2").SetType(&SerialType{T: "serial", SequenceName: "previous_name"}),
+							To:     schema.NewIntColumn("c2", "integer"),
+							Change: schema.ChangeType,
+						},
+					},
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `ALTER TABLE "public"."posts" ALTER COLUMN "c1" DROP DEFAULT, ALTER COLUMN "c1" TYPE integer, ALTER COLUMN "c2" DROP DEFAULT`,
+						Reverse: `ALTER TABLE "public"."posts" ALTER COLUMN "c2" SET DEFAULT nextval('"public"."previous_name"'), ALTER COLUMN "c1" SET DEFAULT nextval('"public"."posts_c1_seq"'), ALTER COLUMN "c1" TYPE smallint`,
+					},
+					{
+						Cmd:     `DROP SEQUENCE IF EXISTS "public"."posts_c1_seq"`,
+						Reverse: `CREATE SEQUENCE IF NOT EXISTS "public"."posts_c1_seq" OWNED BY "public"."posts"."c1"`,
+					},
+					{
+						Cmd:     `DROP SEQUENCE IF EXISTS "public"."previous_name"`,
+						Reverse: `CREATE SEQUENCE IF NOT EXISTS "public"."previous_name" OWNED BY "public"."posts"."c2"`,
+					},
+				},
+			},
+		},
+		// Add serial sequence.
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+							schema.NewColumn("c2").SetType(&SerialType{T: "bigserial"}),
+						),
+					Changes: schema.Changes{
+						&schema.ModifyColumn{
+							From:   schema.NewIntColumn("c1", "integer"),
+							To:     schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+							Change: schema.ChangeType,
+						},
+						&schema.ModifyColumn{
+							From:   schema.NewIntColumn("c2", "integer"),
+							To:     schema.NewColumn("c2").SetType(&SerialType{T: "bigserial"}),
+							Change: schema.ChangeType,
+						},
+					},
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `CREATE SEQUENCE IF NOT EXISTS "public"."posts_c1_seq" OWNED BY "public"."posts"."c1"`,
+						Reverse: `DROP SEQUENCE IF EXISTS "public"."posts_c1_seq"`,
+					},
+					{
+						Cmd:     `CREATE SEQUENCE IF NOT EXISTS "public"."posts_c2_seq" OWNED BY "public"."posts"."c2"`,
+						Reverse: `DROP SEQUENCE IF EXISTS "public"."posts_c2_seq"`,
+					},
+					{
+						Cmd:     `ALTER TABLE "public"."posts" ALTER COLUMN "c1" SET DEFAULT nextval('"public"."posts_c1_seq"'), ALTER COLUMN "c2" SET DEFAULT nextval('"public"."posts_c2_seq"'), ALTER COLUMN "c2" TYPE bigint`,
+						Reverse: `ALTER TABLE "public"."posts" ALTER COLUMN "c2" DROP DEFAULT, ALTER COLUMN "c2" TYPE integer, ALTER COLUMN "c1" DROP DEFAULT`,
+					},
+				},
+			},
+		},
+		// Change underlying sequence type.
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+							schema.NewColumn("c2").SetType(&SerialType{T: "bigserial"}),
+						),
+					Changes: schema.Changes{
+						&schema.ModifyColumn{
+							From:   schema.NewColumn("c1").SetType(&SerialType{T: "smallserial"}),
+							To:     schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+							Change: schema.ChangeType,
+						},
+						&schema.ModifyColumn{
+							From:   schema.NewColumn("c2").SetType(&SerialType{T: "serial"}),
+							To:     schema.NewColumn("c2").SetType(&SerialType{T: "bigserial"}),
+							Change: schema.ChangeType,
+						},
+					},
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `ALTER TABLE "public"."posts" ALTER COLUMN "c1" TYPE integer, ALTER COLUMN "c2" TYPE bigint`,
+						Reverse: `ALTER TABLE "public"."posts" ALTER COLUMN "c2" TYPE integer, ALTER COLUMN "c1" TYPE smallint`,
+					},
+				},
+			},
+		},
+		// Empty qualifier.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("test1")).
+						AddColumns(
+							schema.NewEnumColumn("c1", schema.EnumName("enum"), schema.EnumValues("a"), schema.EnumSchema(schema.New("test2"))),
+						),
+				},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			mock: func(m mock) {
+				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e'")).
+					WithArgs("enum").
+					WillReturnRows(sqlmock.NewRows([]string{"name"}))
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `CREATE TYPE "enum" AS ENUM ('a')`,
+						Reverse: `DROP TYPE "enum"`,
+					},
+					{
+						Cmd:     `CREATE TABLE "posts" ("c1" "enum" NOT NULL)`,
+						Reverse: `DROP TABLE "posts"`,
+					},
+				},
+			},
+		},
+		// Empty sequence qualifier.
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+						),
+					Changes: schema.Changes{
+						&schema.ModifyColumn{
+							From:   schema.NewIntColumn("c1", "integer"),
+							To:     schema.NewColumn("c1").SetType(&SerialType{T: "serial"}),
+							Change: schema.ChangeType,
+						},
+					},
+				},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `CREATE SEQUENCE IF NOT EXISTS "posts_c1_seq" OWNED BY "posts"."c1"`,
+						Reverse: `DROP SEQUENCE IF EXISTS "posts_c1_seq"`,
+					},
+					{
+						Cmd:     `ALTER TABLE "posts" ALTER COLUMN "c1" SET DEFAULT nextval('"posts_c1_seq"')`,
+						Reverse: `ALTER TABLE "posts" ALTER COLUMN "c1" DROP DEFAULT`,
+					},
+				},
+			},
+		},
+		// Empty index qualifier.
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("posts").
+						SetSchema(schema.New("public")).
+						AddColumns(
+							schema.NewIntColumn("c", "int"),
+						),
+					Changes: schema.Changes{
+						&schema.AddIndex{
+							I: schema.NewIndex("i").AddColumns(schema.NewIntColumn("c", "int")),
+						},
+					},
+				},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `CREATE INDEX "i" ON "posts" ("c")`,
+						Reverse: `DROP INDEX "i"`,
+					},
+				},
+			},
+		},
+		// Empty qualifier in multi-schema mode should fail.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{T: schema.NewTable("t1").SetSchema(schema.New("s1")).AddColumns(schema.NewIntColumn("a", "int"))},
+				&schema.AddTable{T: schema.NewTable("t2").SetSchema(schema.New("s2")).AddColumns(schema.NewIntColumn("a", "int"))},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			wantErr: true,
+		},
 	}
 	for i, tt := range tests {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
@@ -793,13 +1120,17 @@ func TestPlanChanges(t *testing.T) {
 			}
 			drv, err := Open(db)
 			require.NoError(t, err)
-			plan, err := drv.PlanChanges(context.Background(), "plan", tt.changes)
+			plan, err := drv.PlanChanges(context.Background(), "wantPlan", tt.changes, tt.options...)
+			if tt.wantErr {
+				require.Error(t, err, "expect plan to fail")
+				return
+			}
 			require.NoError(t, err)
-			require.Equal(t, tt.plan.Reversible, plan.Reversible)
-			require.Equal(t, tt.plan.Transactional, plan.Transactional)
+			require.Equal(t, tt.wantPlan.Reversible, plan.Reversible)
+			require.Equal(t, tt.wantPlan.Transactional, plan.Transactional)
 			for i, c := range plan.Changes {
-				require.Equal(t, tt.plan.Changes[i].Cmd, c.Cmd)
-				require.Equal(t, tt.plan.Changes[i].Reverse, c.Reverse)
+				require.Equal(t, tt.wantPlan.Changes[i].Cmd, c.Cmd)
+				require.Equal(t, tt.wantPlan.Changes[i].Reverse, c.Reverse)
 			}
 		})
 	}
