@@ -6,11 +6,14 @@ package postgres
 
 import (
 	"fmt"
+	"strconv"
 	"testing"
 
+	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/internal/spectest"
+	"ariga.io/atlas/sql/internal/specutil"
 	"ariga.io/atlas/sql/schema"
-
+	"ariga.io/atlas/sql/sqlspec"
 	"github.com/stretchr/testify/require"
 )
 
@@ -93,11 +96,12 @@ table "accounts" {
 }
 
 enum "account_type" {
+	schema = schema.schema
 	values = ["private", "business"]
 }
 `
 	var s schema.Schema
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 	exp := &schema.Schema{
 		Name: "schema",
@@ -211,6 +215,7 @@ enum "account_type" {
 						Type: &schema.EnumType{
 							T:      "account_type",
 							Values: []string{"private", "business"},
+							Schema: exp,
 						},
 					},
 				},
@@ -255,6 +260,7 @@ enum "account_type" {
 			{SeqNo: 0, C: exp.Tables[1].Columns[0]},
 		},
 	}
+	exp.Realm = schema.NewRealm(exp)
 	require.EqualValues(t, exp, &s)
 }
 
@@ -274,7 +280,7 @@ table "t" {
 `
 	t.Run("Invalid", func(t *testing.T) {
 		f := fmt.Sprintf(f, "UNK")
-		err := UnmarshalHCL([]byte(f), &schema.Schema{})
+		err := EvalHCLBytes([]byte(f), &schema.Schema{}, nil)
 		require.Error(t, err)
 	})
 	t.Run("Valid", func(t *testing.T) {
@@ -282,7 +288,7 @@ table "t" {
 			s schema.Schema
 			f = fmt.Sprintf(f, "HASH")
 		)
-		err := UnmarshalHCL([]byte(f), &s)
+		err := EvalHCLBytes([]byte(f), &s, nil)
 		require.NoError(t, err)
 		idx := s.Tables[0].Indexes[0]
 		require.Equal(t, IndexTypeHash, idx.Attrs[0].(*IndexType).T)
@@ -305,7 +311,7 @@ table "t" {
 }
 `
 	var s schema.Schema
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
 	idx := s.Tables[0].Indexes[0]
 	require.Equal(t, IndexTypeBRIN, idx.Attrs[0].(*IndexType).T)
@@ -332,11 +338,12 @@ table "logs" {
 }
 `
 		)
-		err := UnmarshalHCL([]byte(f), s)
+		err := EvalHCLBytes([]byte(f), s, nil)
 		require.NoError(t, err)
 		c := schema.NewStringColumn("name", "text")
 		expected := schema.New("test").
 			AddTables(schema.NewTable("logs").AddColumns(c).AddAttrs(&Partition{T: PartitionTypeHash, Parts: []*PartitionPart{{C: c}}}))
+		expected.SetRealm(schema.NewRealm(expected))
 		require.Equal(t, expected, s)
 	})
 
@@ -362,16 +369,17 @@ table "logs" {
 }
 `
 		)
-		err := UnmarshalHCL([]byte(f), s)
+		err := EvalHCLBytes([]byte(f), s, nil)
 		require.NoError(t, err)
 		c := schema.NewStringColumn("name", "text")
 		expected := schema.New("test").
 			AddTables(schema.NewTable("logs").AddColumns(c).AddAttrs(&Partition{T: PartitionTypeRange, Parts: []*PartitionPart{{C: c}, {X: &schema.RawExpr{X: "lower(name)"}}}}))
+		expected.SetRealm(schema.NewRealm(expected))
 		require.Equal(t, expected, s)
 	})
 
 	t.Run("Invalid", func(t *testing.T) {
-		err := UnmarshalHCL([]byte(`
+		err := EvalHCLBytes([]byte(`
 			schema "test" {}
 			table "logs" {
 				schema = schema.test
@@ -380,10 +388,10 @@ table "logs" {
 					columns = [column.name]
 				}
 			}
-		`), &schema.Schema{})
+		`), &schema.Schema{}, nil)
 		require.EqualError(t, err, "missing attribute logs.partition.type")
 
-		err = UnmarshalHCL([]byte(`
+		err = EvalHCLBytes([]byte(`
 			schema "test" {}
 			table "logs" {
 				schema = schema.test
@@ -392,10 +400,10 @@ table "logs" {
 					type = HASH
 				}
 			}
-		`), &schema.Schema{})
+		`), &schema.Schema{}, nil)
 		require.EqualError(t, err, `missing columns or expressions for logs.partition`)
 
-		err = UnmarshalHCL([]byte(`
+		err = EvalHCLBytes([]byte(`
 			schema "test" {}
 			table "logs" {
 				schema = schema.test
@@ -406,7 +414,7 @@ table "logs" {
 					by { column = column.name }
 				}
 			}
-		`), &schema.Schema{})
+		`), &schema.Schema{}, nil)
 		require.EqualError(t, err, `multiple definitions for logs.partition, use "columns" or "by"`)
 	})
 }
@@ -580,7 +588,7 @@ table "t" {
 `
 	t.Run("Invalid", func(t *testing.T) {
 		f := fmt.Sprintf(f, "UNK")
-		err := UnmarshalHCL([]byte(f), &schema.Schema{})
+		err := EvalHCLBytes([]byte(f), &schema.Schema{}, nil)
 		require.Error(t, err)
 	})
 	t.Run("Valid", func(t *testing.T) {
@@ -588,7 +596,7 @@ table "t" {
 			s schema.Schema
 			f = fmt.Sprintf(f, "ALWAYS")
 		)
-		err := UnmarshalHCL([]byte(f), &s)
+		err := EvalHCLBytes([]byte(f), &s, nil)
 		require.NoError(t, err)
 		id := s.Tables[0].Columns[0].Attrs[0].(*Identity)
 		require.Equal(t, GeneratedTypeAlways, id.Generation)
@@ -662,9 +670,9 @@ table "users" {
 }
 `
 	)
-	err := UnmarshalHCL([]byte(f), &s)
+	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
-	exp := schema.New("test").
+	expected := schema.New("test").
 		AddTables(
 			schema.NewTable("users").
 				AddColumns(
@@ -673,7 +681,8 @@ table "users" {
 					schema.NewIntColumn("c3", "int").SetGeneratedExpr(&schema.GeneratedExpr{Expr: "3", Type: "STORED"}),
 				),
 		)
-	require.EqualValues(t, exp, &s)
+	expected.SetRealm(schema.NewRealm(expected))
+	require.EqualValues(t, expected, &s)
 }
 
 func TestMarshalSpec_Enum(t *testing.T) {
@@ -710,12 +719,12 @@ table "table2" {
     type = enum.account_type
   }
 }
-schema "test" {
-}
 enum "account_type" {
   schema = schema.test
   values = ["private", "business"]
 }
+schema "test" {
+}
 `
 	require.EqualValues(t, expected, string(buf))
 }
@@ -764,7 +773,7 @@ schema "test" {
 }
 
 func TestTypes(t *testing.T) {
-	// TODO(rotemtam) interval
+	p := func(i int) *int { return &i }
 	for _, tt := range []struct {
 		typeExpr string
 		expected schema.Type
@@ -909,10 +918,42 @@ func TestTypes(t *testing.T) {
 			typeExpr: "timestamptz(4)",
 			expected: typeTime(TypeTimestampTZ, 4),
 		},
+		{
+			typeExpr: "interval",
+			expected: &IntervalType{T: "interval"},
+		},
+		{
+			typeExpr: "interval(1)",
+			expected: &IntervalType{T: "interval", Precision: p(1)},
+		},
+		{
+			typeExpr: "second",
+			expected: &IntervalType{T: "interval", F: "second"},
+		},
+		{
+			typeExpr: "minute_to_second",
+			expected: &IntervalType{T: "interval", F: "minute to second"},
+		},
+		{
+			typeExpr: "minute_to_second(2)",
+			expected: &IntervalType{T: "interval", F: "minute to second", Precision: p(2)},
+		},
 		{
 			typeExpr: "real",
 			expected: &schema.FloatType{T: TypeReal, Precision: 24},
 		},
+		{
+			typeExpr: "float",
+			expected: &schema.FloatType{T: TypeFloat},
+		},
+		{
+			typeExpr: "float(1)",
+			expected: &schema.FloatType{T: TypeFloat, Precision: 1},
+		},
+		{
+			typeExpr: "float(25)",
+			expected: &schema.FloatType{T: TypeFloat, Precision: 25},
+		},
 		{
 			typeExpr: "float8",
 			expected: &schema.FloatType{T: TypeFloat8, Precision: 53},
@@ -925,10 +966,26 @@ func TestTypes(t *testing.T) {
 			typeExpr: "numeric",
 			expected: &schema.DecimalType{T: TypeNumeric},
 		},
+		{
+			typeExpr: "numeric(10)",
+			expected: &schema.DecimalType{T: TypeNumeric, Precision: 10},
+		},
+		{
+			typeExpr: "numeric(10, 2)",
+			expected: &schema.DecimalType{T: TypeNumeric, Precision: 10, Scale: 2},
+		},
 		{
 			typeExpr: "decimal",
 			expected: &schema.DecimalType{T: TypeDecimal},
 		},
+		{
+			typeExpr: "decimal(10)",
+			expected: &schema.DecimalType{T: TypeDecimal, Precision: 10},
+		},
+		{
+			typeExpr: "decimal(10,2)",
+			expected: &schema.DecimalType{T: TypeDecimal, Precision: 10, Scale: 2},
+		},
 		{
 			typeExpr: "smallserial",
 			expected: &SerialType{T: TypeSmallSerial},
@@ -975,27 +1032,39 @@ func TestTypes(t *testing.T) {
 		},
 		{
 			typeExpr: `sql("int[]")`,
-			expected: &ArrayType{T: "int[]"},
+			expected: &ArrayType{Type: &schema.IntegerType{T: "int"}, T: "int[]"},
 		},
 		{
 			typeExpr: `sql("int[2]")`,
-			expected: &ArrayType{T: "int[]"},
+			expected: &ArrayType{Type: &schema.IntegerType{T: "int"}, T: "int[]"},
 		},
 		{
 			typeExpr: `sql("text[][]")`,
-			expected: &ArrayType{T: "text[]"},
+			expected: &ArrayType{Type: &schema.StringType{T: "text"}, T: "text[]"},
 		},
 		{
 			typeExpr: `sql("integer [3][3]")`,
-			expected: &ArrayType{T: "integer[]"},
+			expected: &ArrayType{Type: &schema.IntegerType{T: "integer"}, T: "integer[]"},
 		},
 		{
-			typeExpr: `sql("integer ARRAY[4]")`,
-			expected: &ArrayType{T: "integer[]"},
+			typeExpr: `sql("integer  ARRAY[4]")`,
+			expected: &ArrayType{Type: &schema.IntegerType{T: "integer"}, T: "integer[]"},
 		},
 		{
 			typeExpr: `sql("integer ARRAY")`,
-			expected: &ArrayType{T: "integer[]"},
+			expected: &ArrayType{Type: &schema.IntegerType{T: "integer"}, T: "integer[]"},
+		},
+		{
+			typeExpr: `sql("character varying(255) [1][2]")`,
+			expected: &ArrayType{Type: &schema.StringType{T: "character varying", Size: 255}, T: "character varying(255)[]"},
+		},
+		{
+			typeExpr: `sql("character varying ARRAY[2]")`,
+			expected: &ArrayType{Type: &schema.StringType{T: "character varying"}, T: "character varying[]"},
+		},
+		{
+			typeExpr: `sql("varchar(2) [ 2 ] [  ]")`,
+			expected: &ArrayType{Type: &schema.StringType{T: "varchar", Size: 2}, T: "varchar(2)[]"},
 		},
 	} {
 		t.Run(tt.typeExpr, func(t *testing.T) {
@@ -1010,14 +1079,14 @@ func TestTypes(t *testing.T) {
 schema "test" {
 }
 `, tt.typeExpr)
-			err := UnmarshalHCL([]byte(doc), &test)
+			err := EvalHCLBytes([]byte(doc), &test, nil)
 			require.NoError(t, err)
 			colspec := test.Tables[0].Columns[0]
 			require.EqualValues(t, tt.expected, colspec.Type.Type)
 			spec, err := MarshalHCL(&test)
 			require.NoError(t, err)
 			var after schema.Schema
-			err = UnmarshalHCL(spec, &after)
+			err = EvalHCLBytes(spec, &after, nil)
 			require.NoError(t, err)
 			require.EqualValues(t, tt.expected, after.Tables[0].Columns[0].Type.Type)
 		})
@@ -1114,6 +1183,83 @@ func TestParseType_Time(t *testing.T) {
 	}
 }
 
+func TestFormatType_Interval(t *testing.T) {
+	p := func(i int) *int { return &i }
+	for i, tt := range []struct {
+		typ *IntervalType
+		fmt string
+	}{
+		{
+			typ: &IntervalType{T: "interval"},
+			fmt: "interval",
+		},
+		{
+			typ: &IntervalType{T: "interval", Precision: p(6)},
+			fmt: "interval",
+		},
+		{
+			typ: &IntervalType{T: "interval", Precision: p(3)},
+			fmt: "interval(3)",
+		},
+		{
+			typ: &IntervalType{T: "interval", F: "DAY"},
+			fmt: "interval day",
+		},
+		{
+			typ: &IntervalType{T: "interval", F: "HOUR TO SECOND"},
+			fmt: "interval hour to second",
+		},
+		{
+			typ: &IntervalType{T: "interval", F: "HOUR TO SECOND", Precision: p(2)},
+			fmt: "interval hour to second(2)",
+		},
+		{
+			typ: &IntervalType{T: "interval", F: "DAY TO HOUR", Precision: p(6)},
+			fmt: "interval day to hour",
+		},
+	} {
+		t.Run(strconv.Itoa(i), func(t *testing.T) {
+			f, err := FormatType(tt.typ)
+			require.NoError(t, err)
+			require.Equal(t, tt.fmt, f)
+		})
+	}
+}
+func TestParseType_Interval(t *testing.T) {
+	p := func(i int) *int { return &i }
+	for i, tt := range []struct {
+		typ    string
+		parsed *IntervalType
+	}{
+		{
+			typ:    "interval",
+			parsed: &IntervalType{T: "interval", Precision: p(6)},
+		},
+		{
+			typ:    "interval(2)",
+			parsed: &IntervalType{T: "interval", Precision: p(2)},
+		},
+		{
+			typ:    "interval day",
+			parsed: &IntervalType{T: "interval", F: "day", Precision: p(6)},
+		},
+		{
+			typ:    "interval day to second(2)",
+			parsed: &IntervalType{T: "interval", F: "day to second", Precision: p(2)},
+		},
+		{
+			typ:    "interval day to second (2)",
+			parsed: &IntervalType{T: "interval", F: "day to second", Precision: p(2)},
+		},
+	} {
+		t.Run(strconv.Itoa(i), func(t *testing.T) {
+			p, err := ParseType(tt.typ)
+			require.NoError(t, err)
+			require.Equal(t, tt.parsed, p)
+		})
+	}
+}
+
 func TestRegistrySanity(t *testing.T) {
 	spectest.RegistrySanityTest(t, TypeRegistry, []string{"enum"})
 }
@@ -1121,3 +1267,38 @@ func TestRegistrySanity(t *testing.T) {
 func TestInputVars(t *testing.T) {
 	spectest.TestInputVars(t, EvalHCL)
 }
+
+func TestQualifyReferencedTables(t *testing.T) {
+	var (
+		col       = schema.NewIntColumn("col", "integer")
+		targetCol = schema.NewIntColumn("target_column", "integer")
+		targetTbl = schema.NewTable("target_table").AddColumns(targetCol)
+		realm     = schema.NewRealm(
+			schema.New("target_schema").AddTables(targetTbl),
+			schema.New("sch").
+				AddTables(
+					schema.NewTable("tbl").
+						AddColumns(col).
+						AddForeignKeys(
+							schema.NewForeignKey("col_fk").
+								SetRefTable(targetTbl).
+								AddColumns(col).
+								AddRefColumns(targetCol),
+						),
+				),
+		)
+		tables = []*sqlspec.Table{
+			{
+				Name:   "tbl",
+				Schema: &schemahcl.Ref{V: "$schema.sch"},
+			},
+			{
+				Name:   "target_table",
+				Schema: &schemahcl.Ref{V: "$schema.target_schema"},
+			},
+		}
+	)
+	require.NoError(t, specutil.QualifyReferencedTables(tables, realm))
+	require.Zero(t, tables[0].Qualifier)
+	require.Equal(t, "target_schema", tables[1].Qualifier)
+}
