@@ -18,7 +18,7 @@ import (
 type planApply struct{ conn }
 
 // PlanChanges returns a migration plan for the given schema changes.
-func (p *planApply) PlanChanges(ctx context.Context, name string, changes []schema.Change) (*migrate.Plan, error) {
+func (p *planApply) PlanChanges(ctx context.Context, name string, changes []schema.Change, opts ...migrate.PlanOption) (*migrate.Plan, error) {
 	s := &state{
 		conn: p.conn,
 		Plan: migrate.Plan{
@@ -26,6 +26,14 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 			Reversible:    true,
 			Transactional: true,
 		},
+		PlanOptions: migrate.PlanOptions{
+			// Currently, the driver does not support attached
+			// schemas and assumed the connected schema is "main".
+			SchemaQualifier: new(string),
+		},
+	}
+	for _, o := range opts {
+		o(&s.PlanOptions)
 	}
 	if err := s.plan(ctx, changes); err != nil {
 		return nil, err
@@ -41,8 +49,8 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 // ApplyChanges applies the changes on the database. An error is returned
 // if the driver is unable to produce a plan to it, or one of the statements
 // is failed or unsupported.
-func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change) error {
-	return sqlx.ApplyChanges(ctx, changes, p)
+func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change, opts ...migrate.PlanOption) error {
+	return sqlx.ApplyChanges(ctx, changes, p, opts...)
 }
 
 // state represents the state of a planning. It's not part of
@@ -51,6 +59,7 @@ func (p *planApply) ApplyChanges(ctx context.Context, changes []schema.Change) e
 type state struct {
 	conn
 	migrate.Plan
+	migrate.PlanOptions
 	skipFKs bool
 }
 
@@ -89,7 +98,7 @@ func (s *state) plan(ctx context.Context, changes []schema.Change) (err error) {
 func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 	var (
 		errs []string
-		b    = Build("CREATE TABLE").Ident(add.T.Name)
+		b    = s.Build("CREATE TABLE").Table(add.T)
 	)
 	if sqlx.Has(add.Extra, &schema.IfNotExists{}) {
 		b.P("IF NOT EXISTS")
@@ -125,7 +134,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 	s.append(&migrate.Change{
 		Cmd:     b.String(),
 		Source:  add,
-		Reverse: Build("DROP TABLE").Table(add.T).String(),
+		Reverse: s.Build("DROP TABLE").Table(add.T).String(),
 		Comment: fmt.Sprintf("create %q table", add.T.Name),
 	})
 	if err := s.tableSeq(ctx, add); err != nil {
@@ -137,7 +146,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 // dropTable builds and executes the query for dropping a table from a schema.
 func (s *state) dropTable(drop *schema.DropTable) error {
 	s.skipFKs = true
-	b := Build("DROP TABLE").Ident(drop.T.Name)
+	b := s.Build("DROP TABLE").Ident(drop.T.Name)
 	if sqlx.Has(drop.Extra, &schema.IfExists{}) {
 		b.P("IF EXISTS")
 	}
@@ -166,17 +175,23 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 	if err := s.addTable(ctx, &schema.AddTable{T: &newT}); err != nil {
 		return err
 	}
-	if err := s.copyRows(modify.T, &newT, modify.Changes); err != nil {
+	copied, err := s.copyRows(modify.T, &newT, modify.Changes)
+	if err != nil {
 		return err
 	}
 	// Drop the current table, and rename the new one to its real name.
 	s.append(&migrate.Change{
-		Cmd:     Build("DROP TABLE").Ident(modify.T.Name).String(),
-		Source:  modify,
-		Comment: fmt.Sprintf("drop %q table after copying rows", modify.T.Name),
+		Cmd:    s.Build("DROP TABLE").Ident(modify.T.Name).String(),
+		Source: modify,
+		Comment: fmt.Sprintf("drop %q table %s", modify.T.Name, func() string {
+			if copied {
+				return "after copying rows"
+			}
+			return "without copying rows (no columns)"
+		}()),
 	})
 	s.append(&migrate.Change{
-		Cmd:     Build("ALTER TABLE").Ident(newT.Name).P("RENAME TO").Ident(modify.T.Name).String(),
+		Cmd:     s.Build("ALTER TABLE").Ident(newT.Name).P("RENAME TO").Ident(modify.T.Name).String(),
 		Source:  modify,
 		Comment: fmt.Sprintf("rename temporary table %q to %q", newT.Name, modify.T.Name),
 	})
@@ -187,8 +202,8 @@ func (s *state) renameTable(c *schema.RenameTable) {
 	s.append(&migrate.Change{
 		Source:  c,
 		Comment: fmt.Sprintf("rename a table from %q to %q", c.From.Name, c.To.Name),
-		Cmd:     Build("ALTER TABLE").Table(c.From).P("RENAME TO").Table(c.To).String(),
-		Reverse: Build("ALTER TABLE").Table(c.To).P("RENAME TO").Table(c.From).String(),
+		Cmd:     s.Build("ALTER TABLE").Table(c.From).P("RENAME TO").Table(c.To).String(),
+		Reverse: s.Build("ALTER TABLE").Table(c.To).P("RENAME TO").Table(c.From).String(),
 	})
 }
 
@@ -259,7 +274,7 @@ func (s *state) addIndexes(t *schema.Table, indexes ...*schema.Index) error {
 			}
 			idx.Name = strings.Join(names, "_")
 		}
-		b := Build("CREATE")
+		b := s.Build("CREATE")
 		if idx.Unique {
 			b.P("UNIQUE")
 		}
@@ -275,7 +290,7 @@ func (s *state) addIndexes(t *schema.Table, indexes ...*schema.Index) error {
 		s.append(&migrate.Change{
 			Cmd:     b.String(),
 			Source:  &schema.AddIndex{I: idx},
-			Reverse: Build("DROP INDEX").Ident(idx.Name).String(),
+			Reverse: s.Build("DROP INDEX").Ident(idx.Name).String(),
 			Comment: fmt.Sprintf("create index %q to table: %q", idx.Name, t.Name),
 		})
 	}
@@ -325,11 +340,8 @@ func (s *state) fks(b *sqlx.Builder, fks ...*schema.ForeignKey) {
 	})
 }
 
-func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.Change) error {
-	var (
-		args       []interface{}
-		fromC, toC []string
-	)
+func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.Change) (bool, error) {
+	var fromC, toC []string
 	for _, column := range to.Columns {
 		// Skip generated columns in INSERT as they are computed.
 		if sqlx.Has(column.Attrs, &schema.GeneratedExpr{}) {
@@ -344,7 +356,7 @@ func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.
 					break
 				}
 				if change != nil {
-					return fmt.Errorf("duplicate changes for column: %q: %T, %T", column.Name, change, c)
+					return false, fmt.Errorf("duplicate changes for column: %q: %T, %T", column.Name, change, c)
 				}
 				change = changes[i]
 			case *schema.ModifyColumn:
@@ -352,12 +364,12 @@ func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.
 					break
 				}
 				if change != nil {
-					return fmt.Errorf("duplicate changes for column: %q: %T, %T", column.Name, change, c)
+					return false, fmt.Errorf("duplicate changes for column: %q: %T, %T", column.Name, change, c)
 				}
 				change = changes[i]
 			case *schema.DropColumn:
 				if c.C.Name == column.Name {
-					return fmt.Errorf("unexpected drop column: %q", column.Name)
+					return false, fmt.Errorf("unexpected drop column: %q", column.Name)
 				}
 			}
 		}
@@ -370,12 +382,11 @@ func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.
 		case *schema.ModifyColumn:
 			toC = append(toC, column.Name)
 			if !column.Type.Null && column.Default != nil && change.Change.Is(schema.ChangeNull|schema.ChangeDefault) {
-				fromC = append(fromC, fmt.Sprintf("IFNULL(`%[1]s`, ?) AS `%[1]s`", column.Name))
 				x, err := defaultValue(column)
 				if err != nil {
-					return err
+					return false, err
 				}
-				args = append(args, x)
+				fromC = append(fromC, fmt.Sprintf("IFNULL(`%[1]s`, %s) AS `%[1]s`", column.Name, x))
 			} else {
 				fromC = append(fromC, column.Name)
 			}
@@ -385,13 +396,17 @@ func (s *state) copyRows(from *schema.Table, to *schema.Table, changes []schema.
 			fromC = append(fromC, column.Name)
 		}
 	}
-	stmt := fmt.Sprintf("INSERT INTO `%s` (%s) SELECT %s FROM `%s`", to.Name, identComma(toC), identComma(fromC), from.Name)
-	s.append(&migrate.Change{
-		Cmd:     stmt,
-		Args:    args,
-		Comment: fmt.Sprintf("copy rows from old table %q to new temporary table %q", from.Name, to.Name),
-	})
-	return nil
+	insert := len(toC) > 0
+	if insert {
+		s.append(&migrate.Change{
+			Cmd: fmt.Sprintf(
+				"INSERT INTO `%s` (%s) SELECT %s FROM `%s`",
+				to.Name, identComma(toC), identComma(fromC), from.Name,
+			),
+			Comment: fmt.Sprintf("copy rows from old table %q to new temporary table %q", from.Name, to.Name),
+		})
+	}
+	return insert, nil
 }
 
 // alterTable alters the table with the given changes. Assuming the changes are "alterable".
@@ -414,7 +429,7 @@ func (s *state) alterTable(modify *schema.ModifyTable) error {
 				return err
 			}
 		case *schema.AddColumn:
-			b := Build("ALTER TABLE").Ident(modify.T.Name)
+			b := s.Build("ALTER TABLE").Ident(modify.T.Name)
 			r := b.Clone()
 			if err := s.column(b.P("ADD COLUMN"), change.C); err != nil {
 				return err
@@ -426,7 +441,7 @@ func (s *state) alterTable(modify *schema.ModifyTable) error {
 				Comment: fmt.Sprintf("add column %q to table: %q", change.C.Name, modify.T.Name),
 			})
 		case *schema.RenameColumn:
-			b := Build("ALTER TABLE").Ident(modify.T.Name).P("RENAME COLUMN")
+			b := s.Build("ALTER TABLE").Ident(modify.T.Name).P("RENAME COLUMN")
 			r := b.Clone()
 			s.append(&migrate.Change{
 				Source:  change,
@@ -456,7 +471,7 @@ func (s *state) tableSeq(ctx context.Context, add *schema.AddTable) error {
 	// SQLite tracks the AUTOINCREMENT in the "sqlite_sequence" table that is created and initialized automatically
 	// whenever the first "PRIMARY KEY AUTOINCREMENT" is created. However, rows in this table are populated after the
 	// first insertion to the associated table (name, seq). Therefore, we check if the sequence table and the row exist,
-	// and in case they are not, we insert a new non zero sequence to it.
+	// and in case they are not, we insert a new non-zero sequence to it.
 	rows, err := s.QueryContext(ctx, "SELECT seq FROM sqlite_sequence WHERE name = ?", add.T.Name)
 	if err != nil || !rows.Next() {
 		s.append(&migrate.Change{
@@ -467,9 +482,9 @@ func (s *state) tableSeq(ctx context.Context, add *schema.AddTable) error {
 		})
 	}
 	if rows != nil {
-		err = rows.Close()
+		return rows.Close()
 	}
-	return err
+	return nil
 }
 
 func (s *state) append(c *migrate.Change) {
@@ -514,15 +529,20 @@ func autoincPK(pk *schema.Index) bool {
 }
 
 // Build instantiates a new builder and writes the given phrase to it.
-func Build(phrase string) *sqlx.Builder {
-	b := &sqlx.Builder{QuoteChar: '`'}
-	return b.P(phrase)
+func (s *state) Build(phrases ...string) *sqlx.Builder {
+	b := &sqlx.Builder{QuoteChar: '`', Schema: s.SchemaQualifier}
+	return b.P(phrases...)
 }
 
 func defaultValue(c *schema.Column) (string, error) {
 	switch x := c.Default.(type) {
 	case *schema.Literal:
-		return sqlx.SingleQuote(x.V)
+		switch c.Type.Type.(type) {
+		case *schema.BoolType, *schema.DecimalType, *schema.IntegerType, *schema.FloatType:
+			return x.V, nil
+		default:
+			return sqlx.SingleQuote(x.V)
+		}
 	case *schema.RawExpr:
 		return x.X, nil
 	default:
